/* 
* Copyright 1998-2014 VIA Technologies, Inc. All Rights Reserved.
* Copyright 2001-2014 S3 Graphics, Inc. All Rights Reserved.
* Copyright 2013-2014 Shanghai Zhaoxin Semiconductor Co., Ltd. All Rights Reserved.
*
* This file is part of zx.ko
* 
* zx.ko is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* zx.ko is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef __KERNEL_INTERFACE_H__
#define __KERNEL_INTERFACE_H__

#include "s3g_def.h"
#include "s3g_types.h"
#include "core_errno.h"
#include "kernel_import.h"

extern void* krnl_init_adapter(void *pdev, int reserved_vmem, int index, krnl_import_func_list_t *import);

extern void  krnl_deinit_adapter(void *data);

extern void  krnl_dump_resource(void *data, int index, int iga_index);

extern void  krnl_save_force_dvfs_index(void *data, int index);

extern void krnl_dump_sensor_temprature(void *data, unsigned int interval);

extern int krnl_force_dvfs_index(void *data, unsigned int value);

extern int krnl_force_dvfs_video_index(void *data, unsigned int value);

extern void  krnl_debugfs_dump(struct os_seq_file *seq_file, void *data, int type, void* arg);


extern void  krnl_notify_vesa_mode(void *data, s3g_mode_info_t *vesa_mode);

extern void krnl_free_vesafb_tempbuffer(void *data);

/*
** For resource final clean up
** Especially for application exit abnormally
** This function will be responsible for recall
** all the resources.
*/

extern void krnl_final_cleanup(void *data, unsigned int gpu_device);


/*
** 2D client related kernel interface
*/

extern unsigned int krnl_create_2ddevice(void *adapter, void *process_context, s3g_graphic_client_type_t type);
extern int krnl_destroy_2ddevice(void *adapter, void *process_context);

extern void krnl_wait_chip_idle(void *adapter);

extern void krnl_kickoff_2d_draw(void *adapter, s3g_2d_draw_t *draw_2d);

extern int  krnl_acquire_aperture(void *adapter, s3g_acquire_aperture_t *acquire_aperture);

extern void krnl_release_aperture(void *adapter, s3g_release_aperture_t *release_aperture);

extern void krnl_get_iga_info(void *adapter, s3g_iga_info_t *iga_info);

extern unsigned int krnl_get_iga_allocation(void *data, int iga_index);

extern void krnl_notify_resize_fb(void *data, unsigned int old_allocation, unsigned int new_allocation);

extern int  krnl_enum_sub_device(void *data);

extern void krnl_wait_allocation_idle(void *data, s3g_wait_allocation_idle_t *wait_allocation);

extern int  krnl_enter_s3g_graphics_mode(void *data, void *filp);

extern int  krnl_leave_s3g_graphics_mode(void *data, void *filp);

extern int krnl_get_allocation_state(void *data, s3g_get_allocation_state_t *state);
/*
** Display manager kernel interface
*/

extern void krnl_query_display_caps(void *data, s3g_display_caps_t *display_caps);
extern void krnl_query_output_caps(void *adapter, s3g_output_caps_t *output_caps, int from_user_space);
extern void krnl_get_device_comb(void* adapter, s3g_dev_comb_t*  dev_comb);
extern void krnl_get_dev_monitor_type(void* adapter, s3g_monitor_type_t*  mon_type);
extern void krnl_get_hpd_info(void* adapter, s3g_hpd_info_t*  hpd_info);

extern void krnl_set_mode(void *adapter, s3g_mode_info_t *mode);
extern void krnl_register_mhl_callback(void *data);
extern int  krnl_get_hdmi_audio_format(void *data, s3g_hdmi_audio_formats *audio_formats);
extern void krnl_set_ps(void *adapter, s3g_ps_info_t *ps);
extern void krnl_set_cursor(void *adapter, s3g_set_cursor_t *cursor_set);
extern void krnl_set_gamma(void *adapter, s3g_gamma_info_t *gamma);
extern void krnl_set_dpms(void *adapter, s3g_dpms_set_t *dpms_set);

extern void  krnl_set_interactive(void* adapter, int active);

extern int krnl_get_mode_timing(void *adapter, s3g_mode_timing_info_t *timing_info);
extern int krnl_check_device_combination(void *adapter, s3g_check_device_combination_t *device_combination);
extern unsigned int krnl_get_iga_mask_per_allocation(void *data, unsigned int hAllocation);
extern int krnl_send_dsi_cmd(void *data, s3g_dsi_cmd_t *dsi_cmd);
extern int krnl_mhl_cbus_send_command(void *data, s3g_cbus_cmd_t *cbus_cmd);

/*
**  power management kernel interface
*/
extern void krnl_save_state(void *adapter, int need_save_memory);
extern int  krnl_restore_state(void *adapter);

extern int  krnl_pwm_func_ctrl(void *data, s3g_pwm_func_ctrl_t *pwm_func_ctrl);

extern void  krnl_brightness_set(void *data, unsigned int brightness);
extern unsigned int  krnl_brightness_get(void *data);
extern void krnl_cabc_set(void *data, unsigned int cabc);

extern void  krnl_query_brightness_caps(void *data, s3g_brightness_caps_t *brightness_caps);

extern int    krnl_get_dbg_level(void* data);
extern void  krnl_set_dbg_level(void* data, int dbg_level);


extern void krnl_get_map_allocation_info(void *data, unsigned int hAllocation, s3g_map_argu_t *map);


/*
** interrupt related interface
*/

extern int   krnl_get_isr_func(void *adapter, void **isr_func, void **isr_bf_func);
extern void  krnl_enable_msi(void *data);
extern int   krnl_disable_interrupt(void *data);
extern int   krnl_enable_interrupt(void *data);
extern int   krnl_wait_event(void *data, s3g_wait_func_t *wait);

extern int  krnl_cec_ctl(void *data, s3g_cec_ctl_t *cec_ctl);

extern int  krnl_create_stream(void *adapter, s3g_create_stream_t *create_stream);
extern void krnl_destroy_stream(void *adapter, s3g_destroy_stream_t *destroy_stream);
extern int  krnl_display_frame(void *adapter, s3g_display_frame_t *display_frame);

extern unsigned char krnl_get_video_hardware_capability(void *data);
extern int krnl_get_address_config(void *data, s3g_get_address_config_t *address_config);
extern int krnl_query_info(void* data, s3g_query_info_t *info);

#define krnl_config_chip krnl_query_info

extern int  krnl_flip(void *data, s3g_flip_info_t *flip);
extern int  krnl_present(void *data, s3g_present_t *present, s3g_clipnotify_t *clip);

typedef struct
{
    unsigned int create_device  :1;  /* out */
    unsigned int destroy_device :1;  /* out */
    unsigned int hDevice;            /* in/out */
    void         *filp;              /* in */
}krnl_hint_t;

extern int krnl_escape_call(void *data, krnl_hint_t *hint, s3g_escape_call_t *escape);
extern int krnl_escape_call_compat(void* adapter, krnl_hint_t *hint,  s3g_escape_call_t *escape);

/*
 * SDK interface
 */
extern int  krnl_create_device(void *data, void *filp, unsigned int *hDevice);
extern void krnl_destroy_device(void *data, unsigned int hDevice);
extern int  krnl_create_allocation(void *data, s3g_create_allocation_t *create_data);
extern void krnl_destroy_allocation(void *data, s3g_destroy_allocation_t *destroy_data);
extern int  krnl_lock_allocation(void *data, s3g_lock_allocation_t *lock_data, enum s3g_mem_space mem_space);
extern void krnl_unlock_allocation(void *data, s3g_unlock_allocation_t *unlock_data, enum s3g_mem_space mem_space);
extern void krnl_flush_allocation(void *data, s3g_flush_allocation_t *flush_data);
extern int  krnl_open_allocation(void *data, s3g_open_allocation_t *open_data);

/*
 * simply accel func
 */
extern int  krnl_fillrect(void *data, unsigned int context_handle, s3g_fillrect_t *fill_rect);
extern int  krnl_copyarea(void *data, unsigned int context_handle, s3g_copyarea_t *copy_area);
extern int  krnl_imageblit(void *data, unsigned int context_handle, s3g_imageblit_t *image_blit);
extern int  krnl_accel_flip(void *data, unsigned int context_handle, s3g_accel_flip_t *flip);
extern void krnl_allocation_inc_refcount(void *data, unsigned int handle);
extern void krnl_destroy_allocation_export(void *data, unsigned int device_handle, unsigned int alloc_handle);


/*
** misc things
*/
#define OS_CALLBACK_POST_EVENT         1
#define OS_CALLBACK_POST_RC_EVENT      2
#define OS_CALLBACK_POST_SYNCOBJ_EVENT 3


extern int krnl_set_callback_func(void *data, int type, void *func, void *argu);

/*
** perf event mgr
*/
extern int krnl_begin_perf_event(void *data, s3g_begin_perf_event_t *begin_perf_event);
extern int krnl_end_perf_event(void *data, s3g_end_perf_event_t *end_perf_event);
extern int krnl_get_perf_event(void *data, s3g_get_perf_event_t *get_event);
extern int krnl_send_perf_event(void *data, s3g_perf_event_t *perf_event);
extern int krnl_get_perf_status(void *data, s3g_perf_status_t *perf_status);
extern int krnl_begin_miu_dump_perf_event(void *data, s3g_begin_miu_dump_perf_event_t *begin_miu_perf_event);
extern int krnl_end_miu_dump_perf_event(void *data, s3g_end_miu_dump_perf_event_t *end_miu_perf_event);
extern int krnl_set_miu_reg_list_perf_event(void *data, s3g_miu_reg_list_perf_event_t *miu_reg_list);
extern int krnl_get_miu_dump_perf_event(void *data, s3g_get_miu_dump_perf_event_t *get_miu_dump);
extern int krnl_direct_get_miu_dump_perf_event(void *data, s3g_direct_get_miu_dump_perf_event_t *direct_get_miu);

/*
** context
*/
extern int krnl_create_context(void *data, s3g_create_context_t *create_context, enum s3g_mem_space mem_space);
extern int krnl_destroy_context(void *data, s3g_destroy_context_t *destroy_context);
extern int krnl_wait_context_idle(void *data, unsigned int context);
extern void krnl_unreference_allocation(void *data, unsigned int handle);
extern void krnl_reference_allocation(void *data, unsigned int handle);

/*
** render
*/
extern int krnl_render(void *data, s3g_render_t *render);

extern int krnl_hdcp_op(void *data, void *filp, s3g_hdcp_op_t *hdcp_op);

/*
** fence sync object
*/
extern int krnl_create_fence_sync_object(void *data, s3g_create_fence_sync_object_t *s3g_create, int binding);
extern int krnl_destroy_fence_sync_object(void *data, s3g_destroy_fence_sync_object_t *s3g_destroy);
extern int krnl_wait_fence_sync_object(void *data, s3g_wait_fence_sync_object_t *s3g_wait);
extern int krnl_fence_value(void *data, s3g_fence_value_t *s3g_value);
extern int krnl_is_fence_object_signaled(void *data, unsigned int fence_sync_object, unsigned long long wait_value);
extern int krnl_flip_limit(void *data, s3g_flip_limit_t *flip);


extern int krnl_create_di_context(void *data, s3g_create_di_context_t *create);
extern int krnl_destroy_di_context(void *data, s3g_destroy_di_context_t *destroy);

extern int krnl_set_power_perf_mode(void *data, s3g_power_perf_mode_info_t *mode);
extern int krnl_begin_end_isp_dvfs(void *data, unsigned int on);

extern void krnl_graphics_client_create(void *data, void *name, s3g_graphic_client_type_t type);

extern void krnl_fill_allocation(void *data, s3g_fill_allocation_t *s3g_fill_data);

extern void krnl_power_off_crtc(void *data);

extern void krnl_set_fb_shadow_for_adapter(void *data, unsigned int handle);

#endif



