/*
* Copyright 1998-2014 VIA Technologies, Inc. All Rights Reserved.
* Copyright 2001-2014 S3 Graphics, Inc. All Rights Reserved.
* Copyright 2013-2014 Shanghai Zhaoxin Semiconductor Co., Ltd. All Rights Reserved.
*
* This file is part of zx.ko
* 
* zx.ko is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* zx.ko is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef __OS_INTERFACE_H__
#define __OS_INTERFACE_H__

#include "kernel_import.h"

#if !defined(S3G_API_CALL)
#define S3G_API_CALL
#endif

extern void s3g_udelay(unsigned long long usecs_num);

extern unsigned long long s3g_begin_timer(void);
extern unsigned long long s3g_end_timer(unsigned long long);

extern unsigned long long s3g_do_div(unsigned long long x, unsigned long long y);

extern int  s3g_find_first_zero_bit(void *buf, unsigned int size);
extern int  s3g_find_next_zero_bit(void *buf, unsigned int size, int offset);
extern void s3g_set_bit(unsigned int nr, void *buf);
extern void s3g_clear_bit(unsigned int nr, void *buf);

extern struct os_atomic *s3g_create_atomic(int val);
extern void s3g_destroy_atomic(struct os_atomic *atomic);
extern int  s3g_atomic_read(struct os_atomic *atomic);
extern void s3g_atomic_inc(struct os_atomic *atomic);
extern void s3g_atomic_dec(struct os_atomic *atomic);

extern struct os_mutex *s3g_create_mutex(void);
extern void  s3g_destroy_mutex(struct os_mutex *mutex);
extern void  s3g_mutex_lock(struct os_mutex *mutex);
extern int   s3g_mutex_lock_killable(struct os_mutex *mutex);
extern int   s3g_mutex_trylock(struct os_mutex *mutex);
extern void  s3g_mutex_unlock(struct os_mutex *mutex);

extern struct os_sema *s3g_create_sema(int value);
extern void  s3g_destroy_sema(struct os_sema *sema);
extern void  s3g_down(struct os_sema *sema);
extern int   s3g_down_trylock(struct os_sema *sema);  
extern void  s3g_up(struct os_sema *sema);

extern struct os_rwsema *s3g_create_rwsema(void);
extern void  s3g_destroy_rwsema(struct os_rwsema *sema);
extern void  s3g_down_read(struct os_rwsema *sema);
extern void  s3g_down_write(struct os_rwsema *sema);
extern void  s3g_up_read(struct os_rwsema *sema);
extern void  s3g_up_write(struct os_rwsema *sema);

extern struct os_spinlock *s3g_create_spinlock(void);
extern void   s3g_destroy_spinlock(struct os_spinlock *spin);
extern void   s3g_spin_lock(struct os_spinlock *spin);
extern void   s3g_spin_unlock(struct os_spinlock *spin);
extern unsigned long s3g_spin_lock_irqsave(struct os_spinlock *spin);
extern void   s3g_spin_unlock_irqrestore(struct os_spinlock *spin, unsigned long flags);

extern int S3G_API_CALL s3g_copy_from_user(void *to, const void *from, unsigned long size);
extern int S3G_API_CALL s3g_copy_to_user(void *to, const void *from, unsigned long size);

extern void* S3G_API_CALL s3g_memset(void *s, int c, unsigned long count);
extern void* S3G_API_CALL s3g_memcpy(void *d, const void *s, unsigned long count);
extern int   S3G_API_CALL s3g_memcmp(const void *, const void *, unsigned long);
extern void  S3G_API_CALL s3g_byte_copy(char* dst, char* src, int len);

extern int   s3g_strcmp(const char *, const char *);
extern char* s3g_strcpy(char *, const char *);
extern int   s3g_strncmp(const char *, const char *, unsigned long);
extern char* s3g_strncpy(char *, const char *, unsigned long);
extern unsigned long s3g_strlen(char *s);

extern void  s3g_slowbcopy_tobus(unsigned char *src, unsigned char *dst, int len);
extern void  s3g_slowbcopy_frombus(unsigned char *src, unsigned char *dst, int len);

extern void s3g_msleep(int);

extern unsigned long s3g_get_current_pid(void);
extern unsigned long s3g_get_current_tid(void);




/****************************** IO access functions*********************************/
extern void S3G_API_CALL s3g_outb(unsigned short port, unsigned char value);
extern char S3G_API_CALL s3g_inb(unsigned short port);
extern unsigned int S3G_API_CALL  s3g_secure_read32(unsigned long addr);
extern unsigned int S3G_API_CALL  s3g_read32(void *addr);
extern unsigned short S3G_API_CALL s3g_read16(void *addr);
extern unsigned char S3G_API_CALL s3g_read8(void *addr);
extern void S3G_API_CALL  s3g_secure_write32(unsigned long addr, unsigned int val);
extern void S3G_API_CALL  s3g_write32(void *addr, unsigned int val);
extern void S3G_API_CALL  s3g_write16(void *addr, unsigned short val);
extern void S3G_API_CALL  s3g_write8(void *addr, unsigned char val);
extern void S3G_API_CALL s3g_memsetio(void *addr, char c, unsigned int size);
extern void S3G_API_CALL s3g_memcpy_fromio(unsigned int *dest, void *src, unsigned int size);
extern void S3G_API_CALL s3g_memcpy_toio(void *dest, unsigned int *src, unsigned int size);

extern int S3G_API_CALL s3g_vsprintf(char *buf, const char *fmt, ...);
extern int S3G_API_CALL s3g_vsnprintf(char *buf, unsigned long size, const char *fmt, ...);

extern int S3G_API_CALL s3g_sscanf(char *buf, char *fmt, ...);

extern void s3g_dump_stack(void);
extern void S3G_API_CALL s3g_assert(int a);
extern void S3G_API_CALL s3g_printk(unsigned int msglevel, const char* fmt, ...);
extern void S3G_API_CALL s3g_cb_printk(const char* msg);

#ifdef _DEBUG_
#define S3G_MSG_LEVEL S3G_DRV_DEBUG
#define s3g_debug(args...) s3g_printk(S3G_DRV_DEBUG, ##args)
#else
#define S3G_MSG_LEVEL S3G_DRV_INFO
#define s3g_debug(args...)
#endif

#define s3g_emerg(args...)   s3g_printk(S3G_DRV_EMERG, ##args)
#define s3g_error(args...)   s3g_printk(S3G_DRV_ERROR, ##args)
#define s3g_info(args...)    s3g_printk(S3G_DRV_INFO,  ##args)
#define s3g_warning(args...) s3g_printk(S3G_DRV_WARNING, ##args)

extern void  s3g_getsecs(long *secs, long *usecs);
extern void  s3g_get_nsecs(unsigned long long *nsecs);

extern void* s3g_create_thread(s3g_thread_func_t func, void *data, const char *thread_name);
extern void s3g_destroy_thread(void *thread);
extern int  s3g_thread_should_stop(void);

extern struct os_wait_event* s3g_create_event(void);
extern void s3g_destroy_event(struct os_wait_event *event);
extern void s3g_wake_up_event(struct os_wait_event *event);
extern s3g_event_status_t s3g_wait_event(struct os_wait_event *event, int msec);
extern void s3g_thread_wake_up(struct os_wait_event *event);
extern s3g_event_status_t s3g_thread_wait(struct os_wait_event *event, int msec);
extern s3g_event_status_t s3g_wait_event_thread_safe(struct os_wait_event *event, condition_func_t condition, void *argu, int msec);

extern struct os_file *s3g_file_open(const char *path, int flags, unsigned short mode);
extern void s3g_file_close(struct os_file *file);
extern int  s3g_file_read(struct os_file *file, void *buf, unsigned long size, unsigned long long *read_pos);
extern int  s3g_file_write(struct os_file *file, void *buf, unsigned long size);

extern int  s3g_try_to_freeze(void);
extern void s3g_set_freezable(void);
extern void s3g_old_set_freezable(void);
extern void s3g_clear_freezable(void);
extern int  s3g_freezing(void);
extern int  s3g_freezable(void);

/* os private alloc pages func*/
extern struct os_pages_memory* s3g_allocate_pages_memory_priv(int size, int page_size, alloc_pages_flags_t alloc_flags);
extern void s3g_free_pages_memory_priv(struct os_pages_memory *memory);
extern s3g_vm_area_t *s3g_map_pages_memory_priv(void *process_context, s3g_map_argu_t *map);
extern void s3g_unmap_pages_memory_priv(s3g_vm_area_t *vm_area);
extern s3g_vm_area_t *s3g_map_io_memory_priv(void *process_context, s3g_map_argu_t *map);
extern void s3g_unmap_io_memory_priv(s3g_vm_area_t *map);
extern unsigned long long s3g_get_page_phys_address(struct os_pages_memory *memory, int page_num, int *page_size);
extern unsigned long* s3g_acquire_pfns(struct os_pages_memory *memory);
extern void s3g_release_pfns(unsigned long *pfn_table);

extern void s3g_flush_cache(s3g_vm_area_t *vma, struct os_pages_memory* memory, unsigned int offset, unsigned int size);
extern void s3g_inv_cache(s3g_vm_area_t *vma, struct os_pages_memory* memory, unsigned int offset, unsigned int size);
extern int  s3g_mtrr_add(unsigned long base, unsigned long size);
extern int  s3g_mtrr_del(int reg, unsigned long base, unsigned long size);

/* os private malloc func */
extern void* s3g_malloc_priv(unsigned long size);
extern void* s3g_calloc_priv(unsigned long size);
extern void  s3g_free_priv(void *addr);

extern int s3g_get_command_status16(void *dev, unsigned short *command);
extern int s3g_get_command_status32(void *dev, unsigned int *command);
extern int s3g_write_command_status16(void *dev, unsigned short command);
extern int s3g_write_command_status32(void *dev, unsigned int command);
extern int s3g_get_rom_save_addr(void *dev, unsigned int *romsave);
extern int s3g_write_rom_save_addr(void *dev, unsigned int romsave);
extern int s3g_get_bar1(void *dev, unsigned int *bar1);
//gpio
extern int S3G_API_CALL s3g_gpio_getvalue(unsigned int gpio, unsigned int type);
extern void S3G_API_CALL s3g_gpio_setvalue(unsigned int gpio, unsigned int type, unsigned int value);
extern int S3G_API_CALL s3g_gpio_request(unsigned int gpio, unsigned int type);
extern void S3G_API_CALL s3g_gpio_free(unsigned int gpio, unsigned int type);
extern int S3G_API_CALL s3g_gpio_direction_input(unsigned int gpio, unsigned int type);
extern int S3G_API_CALL s3g_gpio_direction_output(unsigned int gpio, unsigned int type, unsigned int value);

//regulator
extern void* s3g_regulator_get(void *pdev, const char *id);
extern int s3g_regulator_enable(void *regulator);
extern int s3g_regulator_disable(void *regulator);
extern int s3g_regulator_is_enabled(void *regulator);
extern int s3g_regulator_get_voltage(void *regulator);
extern int s3g_regulator_set_voltage(void *regulator, int min_uV, int max_uV);
extern void s3g_regulator_put(void *regulator);

extern void s3g_get_bus_config(void *pdev, bus_config_t *bus);
extern int s3g_get_platform_config(void *dev, const char* config_name, int *buffer, int length);


extern unsigned long s3g_get_rom_start_addr(void *dev);
extern void         *s3g_ioremap( unsigned int io_base, unsigned int size);
extern void          s3g_iounmap(void *map_address);
/* mem track func*/
extern void s3g_mem_track_init(void);
extern void s3g_mem_track_list_result(void);

/* alloc pages track func*/
extern struct os_pages_memory *s3g_allocate_pages_memory_track(int size, int page_size,
                                   alloc_pages_flags_t alloc_flags, 
                                   const char *file, unsigned int line);
extern void s3g_free_pages_memory_track(struct os_pages_memory *memory, 
                                        const char *file, unsigned int line);

extern s3g_vm_area_t *s3g_map_pages_memory_track(void *process_context, 
                                        s3g_map_argu_t *map, 
                                        const char *file, unsigned int line);
extern void s3g_unmap_pages_memory_track(s3g_vm_area_t *vm_area, 
                                         const char *file, unsigned int line);

extern s3g_vm_area_t *s3g_map_io_memory_track(void *process_context, 
                                     s3g_map_argu_t *map,
                                     const char *file, unsigned int line);
extern void s3g_unmap_io_memory_track(s3g_vm_area_t *vm_area,
                                      const char *file, unsigned int line);

/* malloc track func*/
extern void* s3g_malloc_track(unsigned long size, const char *file, unsigned int line);
extern void* s3g_calloc_track(unsigned long size, const char *file, unsigned int line);
extern void  s3g_free_track(void *addr, const char *file, unsigned int line);

extern void  s3g_mem_leak_list(void);

#if S3G_MALLOC_TRACK
#define s3g_malloc(size)         s3g_malloc_track(size, __FILE__, __LINE__)
#define s3g_calloc(size)         s3g_calloc_track(size, __FILE__, __LINE__)
#define s3g_free(addr)           s3g_free_track(addr, __FILE__, __LINE__)
#else
#define s3g_malloc(size)         s3g_malloc_priv(size)
#define s3g_calloc(size)         s3g_calloc_priv(size)
#define s3g_free(addr)           s3g_free_priv(addr)
#endif

#if S3G_ALLOC_PAGE_TRACK
#define s3g_allocate_pages_memory(size, flag) \
        s3g_allocate_pages_memory_track(size, flag, __FILE__, __LINE__)
#define s3g_free_pages_memory(memory)         \
        s3g_free_pages_memory_track(memory, __FILE__, __LINE__) 
#else
#define s3g_allocate_pages_memory(size, flag) \
        s3g_allocate_pages_memory_priv(size, flag)
#define s3g_free_pages_memory(memory)         \
        s3g_free_pages_memory_priv(memory)
#endif

#if S3G_MAP_PAGES_TRACK
#define s3g_map_pages_memory(priv, argu)     \
        s3g_map_pages_memory_track(priv, argu, __FILE__, __LINE__)
#define s3g_unmap_pages_memory(map) \
        s3g_unmap_pages_memory_track(map, __FILE__, __LINE__)
#else
#define s3g_map_pages_memory(priv, argu)     \
        s3g_map_pages_memory_priv(priv, argu)
#define s3g_unmap_pages_memory(vma) \
        s3g_unmap_pages_memory_priv(vma)
#endif

#if S3G_MAP_IO_TRACK
#define s3g_map_io_memory(priv, argu) \
        s3g_map_io_memory_track(priv, argu, __FILE__, __LINE__)
#define s3g_unmap_io_memory(argu) \
        s3g_unmap_io_memory_track(argu, __FILE__, __LINE__)
#else
#define s3g_map_io_memory(priv, argu) \
        s3g_map_io_memory_priv(priv, argu)
#define s3g_unmap_io_memory(argu) \
        s3g_unmap_io_memory_priv(argu)
#endif

extern int s3g_get_mem_info(mem_info_t *mem);
extern int s3g_query_platform_caps(platform_caps_t *caps);

extern struct os_shrinker *s3g_register_shrinker(s3g_shrink_callback_t shrink_func, void *shrink_argu);
extern void s3g_unregister_shrinker(struct os_shrinker *s3g_shrinker);

extern int   s3g_pages_memory_swapin(struct os_pages_memory *pages_memory, void *file);
extern void *s3g_pages_memory_swapout(struct os_pages_memory *pages_memory);

void s3g_release_file_storage(void *file);

extern char *s3g_mem_track_result_path;

#ifdef S3G_TRACE_EVENT
extern void s3g_register_trace_events(void);
extern void s3g_unregister_trace_events(void);
extern void s3g_fence_back_trace_event(int engine_index, unsigned long long fence);
extern void s3g_cmd_render_trace_event(int engine_index, void *context, unsigned long long render_counter);
extern void s3g_dma_submit_trace_event(int engine_index, void *context, unsigned long long render_counter, unsigned long long fence);
#endif

extern int s3g_seq_printf(struct os_seq_file *, const char *, ...);

extern void s3g_register_gfx_ts_handle(s3g_gpu_ts_callback_t cb, void *data);

#endif

