/* 
* Copyright 1998-2014 VIA Technologies, Inc. All Rights Reserved.
* Copyright 2001-2014 S3 Graphics, Inc. All Rights Reserved.
* Copyright 2013-2014 Shanghai Zhaoxin Semiconductor Co., Ltd. All Rights Reserved.
*
* This file is part of zx.ko
* 
* zx.ko is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* zx.ko is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef __S3G_DRIVER_H
#define __S3G_DRIVER_H

#include "s3.h"
#include "s3g_def.h"
#include "s3g_types.h"
#include "os_interface.h"
#include "kernel_interface.h"

#define S3G_MAX_CARDS         16


typedef struct s3g_file s3g_file_t;

typedef int (*s3g_ioctl_t)(s3g_file_t* priv, unsigned int cmd, unsigned long arg);

typedef int (*irq_func_t)(void*);

typedef struct
{
    unsigned int               size_in;
    unsigned int               size_out;
    int                        send_data[MAX_IPC_OPERATION_DATA];
    int                        reply_data[MAX_IPC_OPERATION_DATA];
}s3g_ipc_data_t;

typedef struct
{
    void              *adapter;
#ifdef S3G_PCIE_BUS
    struct pci_dev    *pdev;
#else
    struct platform_device *pdev;
#endif
    struct device     *device; //s3g device
    char              busId[64];
    int               len;
    int               index;

    struct os_mutex   *lock;

    int               msi_enable;

    irq_func_t        isr_func;
    void              *isr_bf_func;

    struct tasklet_struct tasklet;

    wait_queue_head_t wait_queue;

    spinlock_t        wait_list_lock;
    struct list_head  wait_list;

    unsigned long long v_sync_count[8];
    unsigned long long v_sync_timestamp[8];

    s3g_iga_virtual_info_t iga_virtual_info[8];

    int               enter_vt;    /* if enter vt means leave console mode */
    unsigned int      s3gfb_pending_events;
    int               s3gfb_enable;
    int               vga16fb_enable;
    int               vesafb_enable;
    int               reserved_vmem;
    s3g_mode_info_t   vesafb_mode;
    int               freezable_patch;

    struct fb_info    *info;
    struct led_classdev lcd;

    s3g_mode_info_t       tv_mode;
    s3g_output_status_t   output_status;

    s3g_ipc_data_t        ipc_data[S3G_IPC_TYPE_COUNT];

    struct s3g_sync_dev  *sync_dev;

    struct input_dev      *rc_dev;
    s3g_keymap_t          *keyboard_rc;
    int                keyboard_rc_keynum;
    void                *debugfs_dev;
}s3g_card_t;

struct s3g_file
{
    struct file       *filp;
    s3g_card_t        *card;

    void              *map;
    unsigned int      gpu_device;

    unsigned int      need_cleanup_drawable;
    unsigned int      server_index;

    int               hold_lock;
    int               freezable;
    struct os_mutex   *lock;

    unsigned int      curr_events;
    unsigned int      wait_events;
    struct list_head  wait_list;
    wait_queue_head_t wait_queue;
};

extern char *s3g_fb_mode;
extern int   s3g_fb;
extern int   s3g_flip;

extern struct class *s3g_class;
extern s3g_card_t    s3g_cards[S3G_MAX_CARDS];
extern s3g_ioctl_t   s3g_ioctls[];
extern s3g_ioctl_t   s3g_ioctls_compat[];

extern int  s3g_card_init(s3g_card_t *s3g, void *pdev);
extern int  s3g_card_deinit(s3g_card_t *s3g);
extern void s3g_card_pre_init(s3g_card_t *s3g, void *pdev);

extern void s3g_interrupt_init(s3g_card_t *s3g);
extern void s3g_interrupt_reinit(s3g_card_t *s3g);
extern void s3g_interrupt_deinit(s3g_card_t *s3g);

extern int  s3g_map_system_io(struct vm_area_struct *vma, s3g_map_argu_t *map);
extern int  s3g_map_system_ram(struct vm_area_struct *vma, s3g_map_argu_t *map);

extern int  s3g_add_wait(s3g_card_t* s3g, s3g_file_t *priv, unsigned int);
extern int  s3g_remove_wait(s3g_card_t* s3g, s3g_file_t* priv);
extern void s3g_get_event(s3g_card_t* s3g, s3g_file_t* priv, s3g_wait_func_t *wait);
extern int  s3g_post_event(s3g_card_t *s3g, unsigned int events);
extern int  s3g_ipc_operation(s3g_card_t *s3g, s3g_file_t *priv, s3g_ipc_operation_t *ipc_operation);

extern void  s3g_init_2d_screen(s3g_card_t *card, s3g_file_t *priv, s3g_2d_screen_t *screen);
extern void  s3g_deinit_2d_screen(s3g_card_t *card, s3g_2d_screen_t *screen);
extern void  s3g_cleanup_2d_screen(s3g_card_t *card, s3g_file_t *priv);

extern int   s3g_fb_probe(s3g_card_t *s3g, char *mode_string, int flip);
extern int   s3g_fb_shutdown(s3g_card_t *s3g);
extern int   s3g_fb_remove(s3g_card_t *s3g);
extern void  s3g_fb_set_pm(s3g_card_t *s3g, int bsuspend);
extern void  s3g_fb_detect_vesa(s3g_card_t *s3g);

#ifdef NEW_S3GFB
extern int   s3gfb_probe(s3g_card_t *s3g, char *mode_string, int flip);
extern int   s3gfb_shutdown(s3g_card_t *s3g);
extern int   s3gfb_remove(s3g_card_t *s3g);
extern void  s3gfb_set_pm(s3g_card_t *s3g, int bsuspend);
extern void  s3gfb_detect_vesa(s3g_card_t *s3g);
extern int   s3gfb_handle_event(s3g_card_t *s3g, unsigned int events);

#define   s3g_fb_probe          s3gfb_probe
#define   s3g_fb_shutdown       s3gfb_shutdown
#define   s3g_fb_remove         s3gfb_remove
#define   s3g_fb_set_pm         s3gfb_set_pm
#define   s3g_fb_detect_vesa    s3gfb_detect_vesa

#else

extern int   s3g_fb_probe(s3g_card_t *s3g, char *mode_string, int flip);
extern int   s3g_fb_shutdown(s3g_card_t *s3g);
extern int   s3g_fb_remove(s3g_card_t *s3g);
extern void  s3g_fb_set_pm(s3g_card_t *s3g, int bsuspend);
extern void  s3g_fb_detect_vesa(s3g_card_t *s3g);

#endif

extern void s3g_init_bus_id(s3g_card_t *s3g);
extern int  s3g_register_driver(void);
extern void s3g_unregister_driver(void);
extern int  s3g_register_interrupt(s3g_card_t *s3g, void *isr);
extern void s3g_unregister_interrupt(s3g_card_t *s3g);

extern int s3g_bo_fd_to_handle(s3g_file_t *priv, s3g_handle_fd_t *hfd);
extern int s3g_bo_handle_to_fd(s3g_file_t *priv, s3g_handle_fd_t *hfd);

extern int  s3g_create_allocation(int card, s3g_create_allocation_t *create_data);
extern void s3g_destroy_allocation(int card, s3g_destroy_allocation_t *destroy_data);
extern int  s3g_lock_allocation(int card, s3g_lock_allocation_t *lock_data);
extern void s3g_unlock_allocation(int card, s3g_unlock_allocation_t *unlock_data);
extern void s3g_flush_allocation(int card, s3g_flush_allocation_t *flush_data);
extern void s3g_open_allocation(int card, s3g_open_allocation_t *open_data);
extern int  s3g_create_device(int card, unsigned int *device);
extern void s3g_destroy_device(int card, unsigned int device);

extern int  s3g_sync_init(s3g_card_t *s3g);
extern void s3g_sync_deinit(s3g_card_t *s3g);
extern void s3g_keyboard_init(s3g_card_t *s3g);

extern void s3g_begin_end_isp_dvfs(int card, unsigned int o);


extern int s3g_ioctl_addctx(s3g_file_t *priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_rmctx(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_lock_ctx(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_unlock_ctx(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_get_busid(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_wait_func(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_get_display_info(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_get_iga_info(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_remove_drawable(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_get_drawsize(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_get_device_comb(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_get_hpd_info(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_query_display_caps(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_set_mode(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_register_mhl_callback(s3g_file_t* priv, unsigned int cmd, unsigned long arg);extern int s3g_ioctl_get_hdmi_audio_format(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_cursor(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_dpms(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_get_mode_timing(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_2d_init(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_wait_chip_idle(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_wait_event(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_acquire_aperture(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_release_aperture(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_leave_vt(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_enter_vt(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_check_device_combination(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_set_ps(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_flip(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_enum_sub_device(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_create_allocation(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_destroy_allocation(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_unlock_allocation(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_flush_allocation(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_set_interactive(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_open_allocation(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_wait_allocation_idle(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_video_get_hw_caps(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_query_info(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_create_stream(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_destroy_stream(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_display_frame(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_cec_ctl(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_present(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_get_drawable_surface(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_get_vsync_cnt(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_create_device(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_destroy_device(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_begin_perf_event(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_begin_miu_dump_perf_event(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_end_perf_event(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_end_miu_dump_perf_event(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_send_perf_event(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_get_perf_status(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_destroy_context(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_create_di_context(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_destroy_di_context(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_query_chip_id(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_hdcp_op(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_create_fence_sync_object(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_destroy_fence_sync_object(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_wait_fence_sync_object(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_fence_value(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_create_fence_fd(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_flip_limit(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_wait_context_idle(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_mhl_cbus_send_command(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_set_power_perf_mode(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_gfx_stm(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_fd_to_handle(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_handle_to_fd(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_get_ust_msc(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_get_allocation_state(s3g_file_t* priv, unsigned int cmd, unsigned long arg);
extern int s3g_ioctl_iga_virtual_info(s3g_file_t *priv, unsigned int cmd, unsigned long arg);
extern void s3g_cleanup_drawable(s3g_file_t *priv);

#endif

