/* 
* Copyright 1998-2014 VIA Technologies, Inc. All Rights Reserved.
* Copyright 2001-2014 S3 Graphics, Inc. All Rights Reserved.
* Copyright 2013-2014 Shanghai Zhaoxin Semiconductor Co., Ltd. All Rights Reserved.
*
* This file is part of zx.ko
* 
* zx.ko is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* zx.ko is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef __S3G_IOCTL_H__
#define __S3G_IOCTL_H__

#if defined(__linux__)
#include <asm/ioctl.h>
#define S3G_IOCTL_NR(n)         _IOC_NR(n)
#define S3G_IOCTL_TYPE(n)       _IOC_TYPE(n)
#define S3G_IOC_VOID            _IOC_NONE
#define S3G_IOC_READ            _IOC_READ
#define S3G_IOC_WRITE           _IOC_WRITE
#define S3G_IOC_READWRITE       _IOC_READ | _IOC_WRITE
#define S3G_IOC(dir, group, nr, size) _IOC(dir, group, nr, size)
#elif defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__) || defined(__DragonFly__)
#if defined(__FreeBSD__) && defined(IN_MODULE)
#undef ioctl
#include <sys/ioccom.h>
#define ioctl(a,b,c)            xf86ioctl(a,b,c)
#else
#include <sys/ioccom.h>
#endif
#define S3G_IOCTL_NR(n)         ((n) & 0xff)
#define S3G_IOCTL_TYPE(n)       (((n) >> 8) & 0xff)
#define S3G_IOC_VOID            IOC_VOID
#define S3G_IOC_READ            IOC_OUT
#define S3G_IOC_WRITE           IOC_IN
#define S3G_IOC_READWRITE       IOC_INOUT
#define S3G_IOC(dir, group, nr, size) _IOC(dir, group, nr, size)
#endif

#ifndef __user
#define __user
#endif

enum s3g_ioctl_nr
{
    ioctl_nr_getbusid,
    ioctl_nr_get_displayinfo,
    ioctl_nr_get_igainfo,  
    ioctl_nr_addctx,
    ioctl_nr_rmctx,
    ioctl_nr_lock,
    ioctl_nr_unlock,
    ioctl_nr_clipnotify,
    ioctl_nr_adddraw,
    ioctl_nr_rmdraw,
    ioctl_nr_getdrawsize,
    ioctl_nr_query_display_caps,
    ioctl_nr_query_output_caps,
    ioctl_nr_mode_set,
    ioctl_nr_cursor,
    ioctl_nr_gamma,
    ioctl_nr_dpms,
    ioctl_nr_get_mode_timing,
    ioctl_nr_leave_vt,
    ioctl_nr_enter_vt,
    ioctl_nr_check_device_combination,
    ioctl_nr_2d_init,
    ioctl_nr_acquire_aperture,
    ioctl_nr_release_aperture,
    ioctl_nr_wait_chip_idle,
    ioctl_nr_escape_call,
    ioctl_nr_2d_draw,
    ioctl_nr_init_screen,
    ioctl_nr_deinit_screen,
    ioctl_nr_set_ps,
    ioctl_nr_enum_sub_device,
    ioctl_nr_create_allocation,
    ioctl_nr_destroy_allocation,
    ioctl_nr_lock_allocation,
    ioctl_nr_unlock_allocation,
    ioctl_nr_open_allocation,
    ioctl_nr_wait_allocation_idle,
    ioctl_nr_video_get_hw_caps,
    ioctl_nr_get_address_config,
    ioctl_nr_query_info,
    ioctl_nr_wait_func,
    ioctl_nr_flip,
    ioctl_nr_cec_ctl,
    ioctl_nr_present,
    ioctl_nr_get_drawable_surface,
    ioctl_nr_get_vsync_cnt,
    ioctl_nr_create_device,
    ioctl_nr_destroy_device,
    ioctl_nr_begin_perf_event,
    ioctl_nr_end_perf_event,
    ioctl_nr_get_perf_event,
    ioctl_nr_send_perf_event,
    ioctl_nr_get_perf_status,
    ioctl_nr_begin_miu_dump_perf_event,
    ioctl_nr_end_miu_dump_perf_event,
    ioctl_nr_set_miu_reg_list_perf_event,
    ioctl_nr_get_miu_dump_perf_event,
    ioctl_nr_direct_get_miu_dump_perf_event,
    ioctl_nr_create_context,
    ioctl_nr_destroy_context,
    ioctl_nr_wait_context_idle,
    ioctl_nr_render,
    ioctl_nr_flush_allocation,
    ioctl_nr_query_chip_id,
    ioctl_nr_hdcp_op,
    ioctl_nr_create_fence_sync_object,
    ioctl_nr_destroy_fence_sync_object,
    ioctl_nr_wait_fence_sync_object,
    ioctl_nr_fence_value,
    ioctl_nr_create_fence_fd, /* it destroy by close(fd)*/
    ioctl_nr_flip_limit,
    ioctl_nr_ipc_operation,
    ioctl_nr_create_stream,
    ioctl_nr_destroy_stream,
    ioctl_nr_display_frame,
    ioctl_nr_set_interactive,
    ioctl_nr_send_dsi_cmd,
    ioctl_nr_mhl_cbus_send_cmd,
    ioctl_nr_create_di_context,
    ioctl_nr_destroy_di_context,
    ioctl_nr_register_mhl_callback,
    ioctl_nr_set_power_perf_mode,
    ioctl_nr_get_device_comb,
    ioctl_nr_get_hpd_info,
    ioctl_nr_gfx_stm,
    ioctl_nr_get_hdmi_audio_format,
    ioctl_nr_wait_event,
    ioctl_nr_handle_to_fd,
    ioctl_nr_fd_to_handle,
    ioctl_nr_get_ust_msc,
    ioctl_nr_get_allocation_state,
    ioctl_nr_video_damage,
    ioctl_nr_iga_virtual_info,
};

/* s3g_fence sync ioctls */
enum s3g_sync_ioctl_nr
{
    ioctl_nr_sync_wait,
    ioctl_nr_sync_merge,
    ioctl_nr_sync_info,
};

enum s3g_buffer_object_ioctl_nr
{
    ioctl_nr_get_read_fence,
    ioctl_nr_get_write_fence,
    ioctl_nr_set_read_fence,
    ioctl_nr_set_write_fence,
    ioctl_nr_get_buffer_data,
    ioctl_nr_set_buffer_data,
};

#define S3G_DIR_NAME "/dev/dri"
#define S3G_DEV_NAME "card"
#define S3G_PROC_NAME "driver/dri"
#define DEV_MEM      "/dev/mem"

#define S3G_IOCTL_BASE                     's'
#define S3G_IO(nr)                         _IO(S3G_IOCTL_BASE, nr)
#define S3G_IOR(nr, type)                  _IOR(S3G_IOCTL_BASE, nr, type)
#define S3G_IOW(nr, type)                  _IOW(S3G_IOCTL_BASE, nr, type)
#define S3G_IOWR(nr, type)                 _IOWR(S3G_IOCTL_BASE, nr, type)

#define S3G_SYNC_IOCTL_BASE                's'
#define S3G_SYNC_IO(nr)                     _IO(S3G_SYNC_IOCTL_BASE, nr)
#define S3G_SYNC_IOR(nr, type)              _IOR(S3G_SYNC_IOCTL_BASE, nr, type)
#define S3G_SYNC_IOW(nr, type)              _IOW(S3G_SYNC_IOCTL_BASE, nr, type)
#define S3G_SYNC_IOWR(nr, type)             _IOWR(S3G_SYNC_IOCTL_BASE, nr, type)

#define S3G_IOCTL_GET_BUSID                S3G_IOWR(ioctl_nr_getbusid, s3g_unique_t)
#define S3G_IOCTL_GET_DISPLAYINFO          S3G_IOWR(ioctl_nr_get_displayinfo, s3g_display_info_t)
#define S3G_IOCTL_GET_IGAINFO              S3G_IOWR(ioctl_nr_get_igainfo, s3g_iga_info_t)
#define S3G_IOCTL_ADD_CTX                  S3G_IOWR(ioctl_nr_addctx, s3g_ctx_t)
#define S3G_IOCTL_RM_CTX                   S3G_IOWR(ioctl_nr_rmctx, s3g_ctx_t)
#define S3G_IOCTL_LOCK                     S3G_IOWR(ioctl_nr_lock, s3g_lock_t)
#define S3G_IOCTL_UNLOCK                   S3G_IOWR(ioctl_nr_unlock, s3g_lock_t)
#define S3G_IOCTL_CLIP_NOTIFY              S3G_IOWR(ioctl_nr_clipnotify, s3g_clipnotify_t)
#define S3G_IOCTL_ADD_DRAW                 S3G_IOWR(ioctl_nr_adddraw, s3g_drawable_t)
#define S3G_IOCTL_RM_DRAW                  S3G_IOWR(ioctl_nr_rmdraw, int)
#define S3G_IOCTL_GET_DRAWSIZE             S3G_IOWR(ioctl_nr_getdrawsize, s3g_get_drawsize_t)
#define S3G_IOCTL_QUERY_DISPLAY_CAPS       S3G_IOWR(ioctl_nr_query_display_caps, s3g_display_caps_t)
#define S3G_IOCTL_QUERY_OUTPUT_CAPS        S3G_IOWR(ioctl_nr_query_output_caps, s3g_output_caps_t)
#define S3G_IOCTL_MODE_SET                 S3G_IOWR(ioctl_nr_mode_set, s3g_mode_info_t)
#define S3G_IOCTL_CURSOR                   S3G_IOWR(ioctl_nr_cursor, s3g_set_cursor_t)
#define S3G_IOCTL_GAMMA                    S3G_IOWR(ioctl_nr_gamma, s3g_gamma_info_t)
#define S3G_IOCTL_DPMS                     S3G_IOWR(ioctl_nr_dpms, s3g_dpms_set_t)
#define S3G_IOCTL_GET_MODE_TIMING          S3G_IOWR(ioctl_nr_get_mode_timing, s3g_mode_timing_info_t)
#define S3G_IOCTL_LEAVE_VT                 S3G_IO(ioctl_nr_leave_vt)
#define S3G_IOCTL_ENTER_VT                 S3G_IO(ioctl_nr_enter_vt)
#define S3G_IOCTL_CHECK_DEVICE_COMBINATION S3G_IOWR(ioctl_nr_check_device_combination, s3g_check_device_combination_t)
#define S3G_IOCTL_2D_INIT                  S3G_IOW(ioctl_nr_2d_init, s3g_2d_init_t)
#define S3G_IOCTL_ACQUIRE_APERTURE         S3G_IOW(ioctl_nr_acquire_aperture, s3g_acquire_aperture_t)
#define S3G_IOCTL_RELEASE_APERTURE         S3G_IOW(ioctl_nr_release_aperture, s3g_release_aperture_t)
#define S3G_IOCTL_WAIT_CHIP_IDLE           S3G_IO(ioctl_nr_wait_chip_idle)
#define S3G_IOCTL_ESCAPE_CALL              S3G_IOW(ioctl_nr_escape_call, long)
#define S3G_IOCTL_2D_DRAW                  S3G_IOW(ioctl_nr_2d_draw, long)
#define S3G_IOCTL_INIT_SCREEN              S3G_IOW(ioctl_nr_init_screen, s3g_2d_screen_t)
#define S3G_IOCTL_DEINIT_SCREEN            S3G_IOW(ioctl_nr_deinit_screen, s3g_2d_screen_t)
#define S3G_IOCTL_SET_PS                   S3G_IOW(ioctl_nr_set_ps, s3g_ps_info_t)
#define S3G_IOCTL_ENUM_SUB_DEVICE          S3G_IOW(ioctl_nr_enum_sub_device, void*)
#define S3G_IOCTL_CREATE_ALLOCATION        S3G_IOWR(ioctl_nr_create_allocation, s3g_create_allocation_t)
#define S3G_IOCTL_DESTROY_ALLOCATION       S3G_IOW(ioctl_nr_destroy_allocation, unsigned int)
#define S3G_IOCTL_IPC_OPERATION            S3G_IOWR(ioctl_nr_ipc_operation, s3g_ipc_operation_t)
#define S3G_IOCTL_SET_INTERACTIVE          S3G_IOWR(ioctl_nr_set_interactive, int)
#define S3G_IOCTL_LOCK_ALLOCATION          S3G_IOWR(ioctl_nr_lock_allocation, s3g_lock_allocation_t)
#define S3G_IOCTL_UNLOCK_ALLOCATION        S3G_IOW(ioctl_nr_unlock_allocation, s3g_unlock_allocation_t)
#define S3G_IOCTL_FLUSH_ALLOCATION         S3G_IOW(ioctl_nr_flush_allocation, s3g_flush_allocation_t)
#define S3G_IOCTL_OPEN_ALLOCATION          S3G_IOWR(ioctl_nr_open_allocation, s3g_open_allocation_t)
#define S3G_IOCTL_WAIT_ALLOCATION_IDLE     S3G_IOW(ioctl_nr_wait_allocation_idle, s3g_wait_allocation_idle_t)
#define S3G_IOCTL_VIDEO_GET_HW_CAPS        S3G_IOWR(ioctl_nr_video_get_hw_caps, unsigned char)
#define S3G_IOCTL_GET_ADDRESS_CONFIG       S3G_IOWR(ioctl_nr_get_address_config, s3g_get_address_config_t)
#define S3G_IOCTL_QUERY_INFO               S3G_IOWR(ioctl_nr_query_info, s3g_query_info_t)
#define S3G_IOCTL_WAIT_FUNC                S3G_IOWR(ioctl_nr_wait_func, s3g_wait_func_t)
#define S3G_IOCTL_FLIP                     S3G_IOW(ioctl_nr_flip, s3g_flip_info_t)
#define S3G_IOCTL_CREATE_STREAM            S3G_IOWR(ioctl_nr_create_stream, s3g_create_stream_t)
#define S3G_IOCTL_DESTROY_STREAM           S3G_IOW(ioctl_nr_destroy_stream, s3g_destroy_stream_t)
#define S3G_IOCTL_DISPLAY_FRAME            S3G_IOWR(ioctl_nr_display_frame, s3g_display_frame_t)
#define S3G_IOCTL_CEC_CTL                  S3G_IOWR(ioctl_nr_cec_ctl, s3g_cec_ctl_t)
#define S3G_IOCTL_PRESENT                  S3G_IOWR(ioctl_nr_present, s3g_present_t)
#define S3G_IOCTL_GET_DRAWABLE_SURFACE     S3G_IOWR(ioctl_nr_get_drawable_surface, s3g_get_drawable_surface_t)
#define S3G_IOCTL_GET_VSYNC_CNT            S3G_IOWR(ioctl_nr_get_vsync_cnt, s3g_get_vsync_cnt_t)
#define S3G_IOCTL_CREATE_DEVICE            S3G_IOWR(ioctl_nr_create_device, s3g_create_device_t)
#define S3G_IOCTL_DESTROY_DEVICE           S3G_IOWR(ioctl_nr_destroy_device, unsigned int)
#define S3G_IOCTL_BEGIN_PERF_EVENT         S3G_IOW(ioctl_nr_begin_perf_event, s3g_begin_perf_event_t)
#define S3G_IOCTL_END_PERF_EVENT           S3G_IOW(ioctl_nr_end_perf_event, s3g_end_perf_event_t)
#define S3G_IOCTL_GET_PERF_EVENT           S3G_IOWR(ioctl_nr_get_perf_event, s3g_get_perf_event_t)
#define S3G_IOCTL_SEND_PERF_EVENT          S3G_IOW(ioctl_nr_send_perf_event, s3g_perf_event_t)
#define S3G_IOCTL_GET_PERF_STATUS          S3G_IOWR(ioctl_nr_get_perf_status, s3g_perf_status_t)
#define S3G_IOCTL_BEGIN_MIU_DUMP_PERF_EVENT S3G_IOWR(ioctl_nr_begin_miu_dump_perf_event, s3g_begin_miu_dump_perf_event_t)
#define S3G_IOCTL_END_MIU_DUMP_PERF_EVENT  S3G_IOWR(ioctl_nr_end_miu_dump_perf_event, s3g_end_miu_dump_perf_event_t)
#define S3G_IOCTL_SET_MIU_REG_LIST_PERF_EVENT S3G_IOWR(ioctl_nr_set_miu_reg_list_perf_event, s3g_miu_reg_list_perf_event_t)
#define S3G_IOCTL_GET_MIU_DUMP_PERF_EVENT  S3G_IOWR(ioctl_nr_get_miu_dump_perf_event, s3g_get_miu_dump_perf_event_t)
#define S3G_IOCTL_DIRECT_GET_MIU_DUMP_PERF_EVENT S3G_IOWR(ioctl_nr_direct_get_miu_dump_perf_event, s3g_direct_get_miu_dump_perf_event_t)
#define S3G_IOCTL_CREATE_CONTEXT           S3G_IOWR(ioctl_nr_create_context, s3g_create_context_t)
#define S3G_IOCTL_DESTROY_CONTEXT          S3G_IOWR(ioctl_nr_destroy_context, s3g_destroy_context_t)
#define S3G_IOCTL_RENDER                   S3G_IOWR(ioctl_nr_render, s3g_render_t)
#define S3G_IOCTL_QUERY_CHIP_ID            S3G_IOWR(ioctl_nr_query_chip_id, s3g_chip_id_t)
#define S3G_IOCTL_HDCP_OP                  S3G_IOWR(ioctl_nr_hdcp_op, s3g_hdcp_op_t)
#define S3G_IOCTL_CREATE_FENCE_SYNC_OBJECT S3G_IOWR(ioctl_nr_create_fence_sync_object, s3g_create_fence_sync_object_t)
#define S3G_IOCTL_DESTROY_FENCE_SYNC_OBJECT S3G_IOWR(ioctl_nr_destroy_fence_sync_object, s3g_destroy_fence_sync_object_t)
#define S3G_IOCTL_WAIT_FENCE_SYNC_OBJECT   S3G_IOWR(ioctl_nr_wait_fence_sync_object, s3g_wait_fence_sync_object_t)
#define S3G_IOCTL_FENCE_VALUE              S3G_IOWR(ioctl_nr_fence_value, s3g_fence_value_t)
#define S3G_IOCTL_SEND_DSI_CMD             S3G_IOWR(ioctl_nr_send_dsi_cmd, s3g_dsi_cmd_t)
#define S3G_IOCTL_MHL_CBUS_SEND_CMD        S3G_IOWR(ioctl_nr_mhl_cbus_send_cmd, s3g_cbus_cmd_t)
#define S3G_IOCTL_CREATE_DI_CONTEXT        S3G_IOWR(ioctl_nr_create_di_context, s3g_create_di_context_t)
#define S3G_IOCTL_DESTROY_DI_CONTEXT       S3G_IOWR(ioctl_nr_destroy_di_context, s3g_destroy_di_context_t)
#define S3G_IOCTL_CREATE_CONTEXT           S3G_IOWR(ioctl_nr_create_context, s3g_create_context_t)
#define S3G_IOCTL_DESTROY_CONTEXT          S3G_IOWR(ioctl_nr_destroy_context, s3g_destroy_context_t)
#define S3G_IOCTL_RENDER                   S3G_IOWR(ioctl_nr_render, s3g_render_t)
#define S3G_IOCTL_GET_DEVICE_COMB          S3G_IOWR(ioctl_nr_get_device_comb, s3g_dev_comb_t)
#define S3G_IOCTL_GET_HPD_INFO             S3G_IOWR(ioctl_nr_get_hpd_info, s3g_hpd_info_t)

#define S3G_IOCTL_CREATE_FENCE_FD          S3G_IOWR(ioctl_nr_create_fence_fd,s3g_create_fence_fd_t)
#define S3G_IOCTL_REGISTER_MHL_CALLBACK    S3G_IO(ioctl_nr_register_mhl_callback)


/*s3g fence sync ioctls*/
#define S3G_SYNC_IOCTL_WAIT     S3G_SYNC_IOWR(ioctl_nr_sync_wait,s3g_wait_fence_t)
#define S3G_SYNC_IOCTL_INFO     S3G_SYNC_IOWR(ioctl_nr_sync_info,s3g_fence_info_t)
#define S3G_SYNC_IOCTL_MERGE    S3G_SYNC_IOWR(ioctl_nr_sync_merge,s3g_fence_merge_t)

#define S3G_IOCTL_SET_FLIP_COUNTER         S3G_IOWR(ioctl_nr_flip_limit,s3g_flip_limit_t)

#define S3G_IOCTL_WAIT_CONTEXT_IDLE        S3G_IOWR(ioctl_nr_wait_context_idle, unsigned int)

#define S3G_IOCTL_SET_READ_FENCE           S3G_IOR(ioctl_nr_set_read_fence, s3g_bo_fence_t)
#define S3G_IOCTL_SET_WRITE_FENCE          S3G_IOR(ioctl_nr_set_write_fence, s3g_bo_fence_t)
#define S3G_IOCTL_GET_READ_FENCE           S3G_IOWR(ioctl_nr_get_read_fence, s3g_bo_fence_t)
#define S3G_IOCTL_GET_WRITE_FENCE          S3G_IOWR(ioctl_nr_get_write_fence, s3g_bo_fence_t)

#define S3G_IOCTL_GET_BUFFER_DATA          S3G_IOWR(ioctl_nr_get_buffer_data, s3g_bo_data_t)
#define S3G_IOCTL_SET_BUFFER_DATA          S3G_IOWR(ioctl_nr_set_buffer_data, s3g_bo_data_t)
#define S3G_IOCTL_GET_HDMI_AUDIO_FORMAT    S3G_IOR(ioctl_nr_get_hdmi_audio_format, s3g_hdmi_audio_formats)
#define S3G_IOCTL_SET_POWER_PERF_MODE      S3G_IOWR(ioctl_nr_set_power_perf_mode, s3g_power_perf_mode_info_t)
#define S3G_IOCTL_GFX_STM                  S3G_IOWR(ioctl_nr_gfx_stm, unsigned int *)
#define S3G_IOCTL_WAIT_EVENT        S3G_IOW(ioctl_nr_wait_event, s3g_wait_func_t)
#define S3G_IOCTL_HANDLE_TO_FD          S3G_IOW(ioctl_nr_handle_to_fd, s3g_handle_fd_t)
#define S3G_IOCTL_FD_TO_HANDLE          S3G_IOW(ioctl_nr_fd_to_handle, s3g_handle_fd_t)
#define S3G_IOCTL_GET_UST_MSC           S3G_IOWR(ioctl_nr_get_ust_msc, s3g_ust_msc_t)
#define S3G_IOCTL_GET_ALLOCATION_STATE           S3G_IOWR(ioctl_nr_get_allocation_state, s3g_get_allocation_state_t)
#define S3G_IOCTL_VIDEO_DAMAGE         S3G_IOW(ioctl_nr_video_damage, s3g_video_damage_t)
#define S3G_IOCTL_IGA_VIRTUAL_INFO         S3G_IOW(ioctl_nr_iga_virtual_info, s3g_iga_virtual_info_t)

/* for fb Driver private ioctl */
/*s3g + index */
#define FB_S3G_IOCTL_GET_ALLOCATION             0x73336701
#define FB_S3G_IOCTL_SET_CURSOR                 0x73336702
#define FB_S3G_IOCTL_QUEUE_POST_ALLOCATION      0x73336703
#define FB_S3G_IOCTL_DEQUEUE_POST_ALLOCATION    0x73336704
#define FB_S3G_IOCTL_GET_FRAMEBUFFER_INFO       0x73336705
#define FB_S3G_IOCTL_SET_ROTATION               0x73336706

#endif //__S3G_IOCTL_H__

