/* 
* Copyright 1998-2014 VIA Technologies, Inc. All Rights Reserved.
* Copyright 2001-2014 S3 Graphics, Inc. All Rights Reserved.
* Copyright 2013-2014 Shanghai Zhaoxin Semiconductor Co., Ltd. All Rights Reserved.
*
* This file is part of zx.ko
* 
* zx.ko is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* zx.ko is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef __S3G_SHARED_H__
#define __S3G_SHARED_H__

#include "s3g_def.h"
#include "s3g_driver.h"

typedef struct framebuffer
{
    int server_index;
    int screen_index;
    int master;
    int samm;
    int mamm;
    int duoview;
    int use_rxa;
    int width;
    int height;
    int depth;
    int tiled;
    unsigned int hdevice;
    unsigned int hAllocation;
    unsigned int rot_allocation[MAX_CORE_CRTCS];
    void *adapter;
    int pitch;
    int gbd;
    unsigned long fbOffset;
    void *priv;
}framebuffer_t;

typedef struct  
{
    int                          xinerama;

    framebuffer_t                *frame_buffers[S3G_MAX_CARDS][MAX_SCREENS];
    framebuffer_t                *frame_shadow_buffers[S3G_MAX_CARDS][MAX_SCREENS];
    
}server_info_t;

extern void s3g_init_2d_screen(s3g_card_t *card, s3g_file_t *priv, s3g_2d_screen_t *screen);
extern void s3g_deinit_2d_screen(s3g_card_t *card, s3g_2d_screen_t *screen);
extern int  s3g_is_xinerama(s3g_card_t *card, int server_index);
extern void s3g_set_server_info(s3g_card_t *card, int server_index, int xinerama);
extern int s3g_get_display_info(s3g_card_t *card, s3g_display_info_t *display_info);
extern void s3g_clip_notify(s3g_card_t *card, s3g_clipnotify_t *clip);
extern void s3g_add_drawable(s3g_card_t *card, s3g_drawable_t *drawable);
extern void s3g_get_drawable_info(s3g_card_t *card, s3g_get_drawsize_t *draw_rec);
extern void s3g_remove_drawable(s3g_card_t *card, int drawId);
extern framebuffer_t* s3g_get_frame_buffer(s3g_card_t *card, int server_index, int screen_index);
extern void s3g_map_iga_to_screen(s3g_card_t *card, int *screens, int server_index);
extern void s3g_map_screen_to_igas(s3g_card_t *card, int *igas, int server_index, int screen_index);
extern s3g_drawable_t *s3g_get_drawable(unsigned int kdraw);
extern void s3g_set_current_fb(s3g_card_t *card, int server_index, int screen_index, framebuffer_t *fb);
extern void s3g_dump_clips(s3g_card_t *card);
extern void s3g_video_drawable_damage(s3g_card_t *card, s3g_video_damage_t *damage);
#endif
