/* 
* Copyright 1998-2014 VIA Technologies, Inc. All Rights Reserved.
* Copyright 2001-2014 S3 Graphics, Inc. All Rights Reserved.
* Copyright 2013-2014 Shanghai Zhaoxin Semiconductor Co., Ltd. All Rights Reserved.
*
* This file is part of zx.ko
* 
* zx.ko is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* zx.ko is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef S3G_FB_H
#define S3G_FB_H

#include "s3g_driver.h"
#include "s3.h"

enum
{
    CHIP_CMODEL,
    CHIP_CLB,
    CHIP_DST,
    CHIP_CSR,
    CHIP_INV,
    CHIP_H5,
    CHIP_H5S1,
    CHIP_H6S2,
    CHIP_CMS,
    CHIP_METRO,
    CHIP_MANHATTAN,
    CHIP_MATRIX,
    CHIP_DST2,
    CHIP_DST3,
    CHIP_DUMA,
    CHIP_H6S1,
    CHIP_DST4,
    CHIP_EXC1,      //Excalibur-1
    CHIP_E2UMA,     //E2UMA
    CHIP_ELT,       //Elite
    CHIP_ELT1K,
    CHIP_ELT2K,
    CHIP_ZX2000,
    CHIP_CHX001,    //CHX001
    CHIP_LAST,
};

typedef struct
{
    unsigned int hAllocation;
    unsigned int fbOffset;
    int gbd;
    int dx;
    int dy;
    int width;
    int height;
    int fg_color;
    int bg_color;
    int depth;
    const char* data;
} s3g_image_t;

typedef struct
{
    unsigned int hAllocation;
    unsigned int xoffset;
    unsigned int yoffset;
    unsigned int pitch;
    unsigned int offset;
    unsigned int gbd;
    unsigned int bpp;
    int pipe;
} s3gfb_set_ps_t;

struct s3gfb_output
{
    unsigned int       type;
    int                connected;
    int                crtc;

    int                num_mode;
    void               *edid;
    void*              modes;

    struct fb_monspecs monspecs;
    struct list_head   modelist;

    const struct fb_videomode *curr_mode; 
};


struct s3gfb_outputs_config
{
    unsigned int        support_outputs;
    unsigned int        connected_outputs;

    int                 num_crtc;
    int                 num_output;

    struct s3gfb_output *outputs;
    struct list_head    *shared_modelist;
};

struct s3gfb_screen
{
    unsigned int x;
    unsigned int y;
    unsigned int width;
    unsigned int height;
    unsigned int bpp;
    int          tiled;

    unsigned int pitch;
    unsigned int gbd;
    unsigned int hAllocation;
    unsigned int fbOffset;
    void         *fbVirtual;
};


struct s3gfb_dev
{
    s3g_card_t     *s3g;
    void           *adapter;
    int            flip;
    char           *mode_option;

    int            disabled;
    int            screen_resize; /* do we can resize screen, if not use default width/height */
    int            default_width;
    int            default_height;
    int            default_bpp;

    int            yoffset;

    unsigned int   fb_start;

    unsigned int   hDevice;

    unsigned int   hContext;

    struct s3gfb_outputs_config outputs_config;

    struct s3gfb_screen   screen;

    unsigned int   pseudo_palette[17];

    int            local_dynamic;

    unsigned int        chip_id;
    int                 stream_type;
    s3g_display_frame_t     display_frame[MAX_CORE_CRTCS];

};

extern struct fb_ops s3gfb_ops;

extern int s3gfb_create_screen(struct s3gfb_dev *dev, struct s3gfb_screen *screen);
extern int s3gfb_destroy_screen(struct s3gfb_dev *dev, struct s3gfb_screen *screen);
extern int s3gfb_set_stream(struct fb_info* info, struct s3gfb_output *output);
extern void s3gfb_destroy_stream(struct fb_info* info);
#endif /*S3G_FB_H*/
