/* 
* Copyright 1998-2014 VIA Technologies, Inc. All Rights Reserved.
* Copyright 2001-2014 S3 Graphics, Inc. All Rights Reserved.
* Copyright 2013-2014 Shanghai Zhaoxin Semiconductor Co., Ltd. All Rights Reserved.
*
* This file is part of zx.ko
* 
* zx.ko is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* zx.ko is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "s3.h"
#include "os_interface.h"
#include "s3g_driver.h"
#include "s3g_ioctl.h"
#include "s3g_version.h"
#include "s3g_debugfs.h"

char *s3g_fb_mode = NULL; //"800x600-32@60";
#ifdef S3G_HW_NULL
int   s3g_fb      = 0;
#else
int   s3g_fb      = 1;
#endif
int   s3g_flip    = 0;

s3g_card_t s3g_cards[S3G_MAX_CARDS];

#if 0
static int s3g_open(struct inode *inode, struct file *filp)
{
    s3g_card_t *s3g = NULL;

    s3g_file_t *priv;

    int minor;

    int err = -ENODEV;

#ifndef MINOR
    minor = minor(inode->i_rdev);
#else
    minor = MINOR(inode->i_rdev);
#endif

    if(!((minor >= 0) && (minor < S3G_MAX_CARDS)))
    {
        return err;
    }

    s3g = &s3g_cards[minor];

    if(!s3g->adapter)
    {
        return err;
    }

    priv = s3g_calloc(sizeof(s3g_file_t));

    if(!priv)
    {
        return -ENOMEM;
    }

    filp->private_data = priv;

    priv->filp        = filp;
    priv->card        = s3g;
    priv->lock        = s3g_create_mutex();

    init_waitqueue_head(&priv->wait_queue);

    return 0;
}

static int s3g_release(struct inode *inode, struct file *filp)
{
    s3g_file_t *priv = filp->private_data;
    s3g_card_t *s3g  = priv->card;

    if(priv->wait_events)
    {
        s3g_remove_wait(s3g, priv);
    }

    if(priv->hold_lock)
    {
        s3g_mutex_unlock(s3g->lock);
    }

    if(priv->gpu_device)
    {
        s3g_cleanup_2d_screen(s3g, priv);

        if(s3g->debugfs_dev)
        {
            s3g_debugfs_remove_node(s3g->debugfs_dev, priv->gpu_device);
        }

        krnl_final_cleanup(s3g->adapter, priv->gpu_device);

        priv->gpu_device = 0;
    }

    s3g_destroy_mutex(priv->lock);
    s3g_free(priv);
    filp->private_data = NULL;

    return 0;
}

static long s3g_unlocked_ioctl(struct file *filp, unsigned int cmd, unsigned long arg)
{
    unsigned int nr   = S3G_IOCTL_NR(cmd);
    s3g_ioctl_t  func = s3g_ioctls[nr];

    long ret;

    ret = (long)func(filp->private_data, cmd, arg);

    return ret;
}


static int s3g_mmap(struct file *filp, struct vm_area_struct *vma)
{
    int ret = 0;
#ifndef __frv__
    s3g_file_t     *priv = filp->private_data;
    s3g_card_t     *card = priv->card;
    s3g_map_argu_t *map  = priv->map;
    s3g_map_argu_t map_argu;

    if(map == NULL)
    {
        bus_config_t bus_config = {0};

        unsigned long offset = vma->vm_pgoff << PAGE_SHIFT;

        s3g_get_bus_config(card->pdev, &bus_config);

        map_argu.flags.mem_type   = S3G_SYSTEM_IO;

        if((offset >= bus_config.mem_start_addr[0]) && 
           (offset <  bus_config.mem_end_addr[0]))
        {
            /* map fb as wc */
            map_argu.flags.cache_type = S3G_MEM_WRITE_COMBINED;
        }
        else
        {
            /* map mmio reg as uc*/
            map_argu.flags.cache_type = S3G_MEM_UNCACHED;
        }

        map = &map_argu;
    }

    switch(map->flags.mem_type)
    {
        case S3G_SYSTEM_IO:
            ret = s3g_map_system_io(vma, map);
            break;
            
        case S3G_SYSTEM_RAM:
            ret = s3g_map_system_ram(vma, map);
            break;

        default:
            s3g_error("%s, unknown memory map type", __func__);
            ret = -1;
            break;
    }
#endif

    return ret;
}

static unsigned int s3g_poll(struct file *filp, struct poll_table_struct *wait)
{
    s3g_file_t *priv = filp->private_data;

    unsigned int mask = 0;

    poll_wait(filp, &priv->wait_queue, wait);

    if(priv->wait_events & priv->curr_events)
    {
        mask = POLLIN;
    }

    return mask;
}

static struct file_operations s3g_fops = 
{
    .owner   = THIS_MODULE,
    .open    = s3g_open,
    .release = s3g_release,
    .poll    = s3g_poll,
    .unlocked_ioctl   = s3g_unlocked_ioctl,
#if defined(__x86_64__) && defined(HAVE_COMPAT_IOCTL)
    .compat_ioctl = s3g_unlocked_ioctl,
#endif
    .mmap = s3g_mmap,
};
#endif

struct class *s3g_class = NULL;

static int __init s3g_init(void)
{
    int ret = -ENOMEM;

    s3g_class = class_create(THIS_MODULE, S3G_DEV_NAME);

    if(s3g_class == NULL)
    {
        s3g_error("class_create() failed in s3g_init.\n");
    }

    s3g_info("Shanghai Zhaoxin Semiconductor Co., Ltd.\n");
    //s3g_info("Version: %0d.%02d.%02d%s Build on: %s\n", DRIVER_MAJOR, DRIVER_MINOR, DRIVER_PATCHLEVEL, DRIVER_BRANCH, DRIVER_DATE);    
    s3g_info("Version: %0d.%02d.%02d Build on: %s\n", DRIVER_MAJOR, DRIVER_MINOR, DRIVER_PATCHLEVEL, DRIVER_DATE);    

#if S3G_MALLOC_TRACK | S3G_ALLOC_PAGE_TRACK | S3G_MAP_PAGES_TRACK | S3G_MAP_IO_TRACK

    s3g_mem_track_init();

#endif
    ret = s3g_register_driver();

    if(ret)
    {
        s3g_error("register_driver() failed in s3g_init. ret:%x.\n", ret);
    }

    return 0;
}

static void __exit s3g_exit(void)
{
    s3g_unregister_driver();

    if(s3g_class != NULL)
    {
        class_destroy(s3g_class);

        s3g_class = NULL;
    }

    unregister_chrdev(S3G_MAJOR, "s3g");

#if S3G_MALLOC_TRACK | S3G_ALLOC_PAGE_TRACK | S3G_MAP_PAGES_TRACK | S3G_MAP_IO_TRACK
    s3g_mem_track_list_result();
#endif

    s3g_info("exit driver.\n");
}

module_init(s3g_init);
module_exit(s3g_exit);
MODULE_LICENSE("GPL");

#ifndef KERNEL_2_4
module_param(s3g_fb, int, 0);
module_param(s3g_fb_mode, charp, 0);
module_param(s3g_flip, int, 0);
#else
MODULE_PARM(s3g_fb_mode, "s");
#endif

MODULE_PARM_DESC(s3g_fb, "enable s3g fb driver");
MODULE_PARM_DESC(s3g_fb_mode, "Initial video mode:<xres>x<yres>-<depth>@<refresh>");
MODULE_PARM_DESC(s3g_flip, "enable frame buffer flip support");

