/* 
* Copyright 1998-2014 VIA Technologies, Inc. All Rights Reserved.
* Copyright 2001-2014 S3 Graphics, Inc. All Rights Reserved.
* Copyright 2013-2014 Shanghai Zhaoxin Semiconductor Co., Ltd. All Rights Reserved.
*
* This file is part of zx.ko
* 
* zx.ko is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* zx.ko is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "s3g_cec.h"

int s3g_post_rc_event(s3g_card_t *s3g, unsigned char ui_cmd_id, int value)
{
    struct input_dev *rc_input;
    int i;

    rc_input = s3g->rc_dev;

    for(i=0; i< s3g->keyboard_rc_keynum; i++)
    {
        if(ui_cmd_id == s3g->keyboard_rc[i].id)
        {
            s3g_info("s3g_post_rc_event: key event, ui_cmd_id and value are 0x%02x, %d \n", ui_cmd_id, value);
            input_event(rc_input, EV_KEY, s3g->keyboard_rc[i].key, value);
            input_sync(rc_input);
            break;
        }
    }

    return 0;
}

int s3g_rc_dev_init(s3g_card_t *s3g)
{
    struct input_dev *rc_input;
    unsigned short *keycode;
    int i, ret;

    rc_input = input_allocate_device();
    if (!rc_input)
    {
        s3g_error("s3g_rc_dev_init: input_allocate_device failed!\n");
        return -1;
    }

    s3g->rc_dev = rc_input;

    keycode = s3g_malloc(s3g->keyboard_rc_keynum * sizeof(unsigned short));
    for (i = 0; i < s3g->keyboard_rc_keynum; i++)
    {
        keycode[i] = s3g->keyboard_rc[i].key;
    }
    rc_input->name = "Elite Remote Control Driver";
    rc_input->keycode = keycode;
    rc_input->keycodesize = sizeof(unsigned short);
    rc_input->keycodemax = s3g->keyboard_rc_keynum;
    rc_input->evbit[0] = BIT_MASK(EV_KEY);
    for (i = 0; i < s3g->keyboard_rc_keynum; i++)
    {
        __set_bit(keycode[i], rc_input->keybit);
    }
    ret = input_register_device(rc_input);
    if(ret)
    {
        s3g_error("s3g_rc_dev_init: input_register_device failed!\n");
        input_free_device(rc_input);
        return -1;
    }

    krnl_set_callback_func(s3g->adapter, OS_CALLBACK_POST_RC_EVENT, s3g_post_rc_event, s3g);

    return 0;
}

void s3g_rc_dev_deinit(s3g_card_t *s3g)
{
    s3g_free(s3g->rc_dev->keycode);
    input_unregister_device(s3g->rc_dev);
    input_free_device(s3g->rc_dev);
}

