/* 
* Copyright 1998-2014 VIA Technologies, Inc. All Rights Reserved.
* Copyright 2001-2014 S3 Graphics, Inc. All Rights Reserved.
* Copyright 2013-2014 Shanghai Zhaoxin Semiconductor Co., Ltd. All Rights Reserved.
*
* This file is part of zx.ko
* 
* zx.ko is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* zx.ko is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include  "s3g_led.h"


static void s3g_brightness_set(struct led_classdev *led_cdev, enum led_brightness brightness)
{
    s3g_card_t * s3g = container_of(led_cdev, s3g_card_t, lcd);
    unsigned int brt = brightness;
    brt = (brt > s3g->lcd.max_brightness)? s3g->lcd.max_brightness : brt;
    krnl_brightness_set(s3g->adapter, brt);
}



static enum led_brightness s3g_brightness_get(struct led_classdev *led_cdev)
{
    s3g_card_t * s3g = container_of(led_cdev, s3g_card_t, lcd);
    unsigned int brt = krnl_brightness_get(s3g->adapter);
    brt = (brt > s3g->lcd.max_brightness)? s3g->lcd.max_brightness : brt;

    return brt;
}

#ifdef  CONFIG_LEDS_CLASS

static ssize_t s3g_cabc_store(struct device *dev,
        struct device_attribute *attr, const char *buf, size_t size)
{
    struct led_classdev *led_cdev = dev_get_drvdata(dev); 
    s3g_card_t * s3g = container_of(led_cdev, s3g_card_t, lcd);
    unsigned long state;
    ssize_t ret = -EINVAL;

    ret = kstrtoul(buf, 10, &state);
    if(!ret)
    {
        krnl_cabc_set(s3g->adapter, state);  
        ret = size;
    }

    return ret;
}

static struct device_attribute cabc_attr=__ATTR(cabc_level_ctl, 0200, NULL,s3g_cabc_store);

#endif

int s3g_led_init(struct platform_device *pdev, s3g_card_t *s3g)
{
    int result = 0;
    s3g_brightness_caps_t caps = {0};
    
    krnl_query_brightness_caps(s3g->adapter, &caps);
    
    s3g->lcd.name           = "lcd-backlight";
    s3g->lcd.brightness_set = s3g_brightness_set;
    s3g->lcd.brightness_get = s3g_brightness_get;    
    s3g->lcd.max_brightness = caps.max_brightness_value;

//for elite2000, LEDS_CLASS will be defined and led module will be built in linux kernel
#ifdef  CONFIG_LEDS_CLASS
    result = led_classdev_register(&pdev->dev, &s3g->lcd);
    if (result < 0) 
    {
       s3g_error("couldn't register lcd device \n");
    }
    result = device_create_file(s3g->lcd.dev, &cabc_attr);
    if(result)
    {
       s3g_error("couldn't create cabc_level_ctl in /sys/class/leds/lcd-backlight/ \n");    
    }
#else
    s3g_warning("Led class is not supported by linux kernel.\n");
#endif
    return result;
}


