/* 
* Copyright 1998-2014 VIA Technologies, Inc. All Rights Reserved.
* Copyright 2001-2014 S3 Graphics, Inc. All Rights Reserved.
* Copyright 2013-2014 Shanghai Zhaoxin Semiconductor Co., Ltd. All Rights Reserved.
*
* This file is part of zx.ko
* 
* zx.ko is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* zx.ko is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "s3.h"
#include "s3g_driver.h"
#include "os_interface.h"
#include "s3g_ioctl.h"
#include "s3g_debugfs.h"
#include "s3g_version.h"

#define DRIVER_NAME         "zx"
#define DRIVER_DESC         "ZX DRM Pro"

int num_probed_card = 0;

#ifdef S3G_GPIO_IRQ
   static int gpio_for_pcie = 0x5;// GPIO PIN for INT
#endif


static struct pci_device_id pciidlist[] =
{
    {0x1106, 0x3A01, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
    {0x1106, 0x3A03, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0}, //chx001 qt
    {0x1d17, 0x3A03, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0}, //chx001
    {0, 0, 0}
};

MODULE_DEVICE_TABLE(pci, pciidlist);

static struct pci_driver s3g_driver;

#if 0
static int s3g_pcie_probe(struct pci_dev *pdev, const struct pci_device_id *ent)
{
    s3g_card_t    *s3g = &s3g_cards[num_probed_card++];
    int           ret  = 0;

    s3g_memset(s3g, 0, sizeof(s3g_card_t));

    pci_set_drvdata(pdev, s3g);

    s3g_card_pre_init(s3g, pdev);

    ret = pci_request_regions(pdev, s3g_driver.name);
    if(ret)
    {
        s3g_error("pci_request_regions() failed. ret:%x.\n", ret);
    }

    ret = pci_enable_device(pdev);
    if(ret)
    {
        s3g_error("pci_enable_device() failed. ret:%x.\n", ret);
    }

    pci_set_master(pdev);
   
    /*don't use the vga arbiter*/    
#if defined(CONFIG_VGA_ARB)
    vga_set_legacy_decoding(pdev, VGA_RSRC_NONE);
#endif

    ret = s3g_card_init(s3g, pdev);

    return ret;
}

static void s3g_pcie_shutdown(struct pci_dev *pdev)
{
    s3g_card_t *s3g = pci_get_drvdata(pdev);

    if(s3g->s3gfb_enable)
    {
#ifdef CONFIG_FB
        s3g_fb_shutdown(s3g);
#endif
    }

}
#endif

static int s3g_drm_suspend(struct drm_device *dev, pm_message_t state)
{
    s3g_card_t* s3g = dev->dev_private;

    s3g_info("s3g driver suspending, pm event = %d.\n", state.event);
#ifndef __mips__
    if(s3g->s3gfb_enable)
    {
#ifdef CONFIG_FB
        s3g_fb_set_pm(s3g, 1);
#endif
    }
    
    krnl_save_state(s3g->adapter, state.event == PM_EVENT_FREEZE);

    /* disable IRQ */
    s3g_interrupt_deinit(s3g);

    pci_save_state(dev->pdev);

    if (state.event == PM_EVENT_SLEEP)
    {
        /*patch for CHX001 A0 and B0, pci_disable_device will clear master bit PCI04[2], and this bit can't be cleared for chx001 hw, so don't call pci_disable_device here*/
        //pci_disable_device(dev->pdev);
        pci_set_power_state(dev->pdev, PCI_D3hot);
    }
#endif

    return 0;
}

static int s3g_drm_resume(struct drm_device *dev)
{
    s3g_card_t* s3g = dev->dev_private;
    int         ret = 0;

    s3g_info("s3g driver resume back.\n");

#ifndef __mips__
    pci_set_power_state(dev->pdev, PCI_D0);

    pci_restore_state(dev->pdev);

    if (pci_enable_device(dev->pdev))
    {
        return -1;
    }
    pci_set_master(dev->pdev);
   
    s3g_interrupt_reinit(s3g); 

    ret = krnl_restore_state(s3g->adapter);

    if(ret != 0)
    {
        return -1;
    }

    if(s3g->s3gfb_enable)
    {
#ifdef CONFIG_FB
    s3g_fb_set_pm(s3g, 0);
#endif
    }
#endif

    return 0;
}

static int s3g_drm_load(struct drm_device *dev, unsigned long flags)
{
    struct pci_dev *pdev = dev->pdev;
    s3g_card_t  *s3g = &s3g_cards[num_probed_card++];
    int         ret = 0;
#if LINUX_VERSION_CODE > KERNEL_VERSION(3,10,52)
    struct dev_pm_ops  *new_pm_ops;

//    drm_debug = 1;

    /* since if drm driver not support DRIVER_MODESET,   it won't support resume from S4, 
        *  because drm_class_dev_pm_ops not set .restore field and won't call into our s3g_drm_resume callback;
        *
        *  so, we should hook the drm_class_dev_pm_ops structure, set  it's .restore field ;
        *  anyway, it's a stupid way to do this
        */
    if (dev && dev->primary && 
        dev->primary->kdev && 
        dev->primary->kdev->class && 
        dev->primary->kdev->class->pm && 
        !dev->primary->kdev->class->pm->restore)
    {
        new_pm_ops = s3g_calloc(sizeof(*new_pm_ops));

        s3g_memcpy(new_pm_ops, dev->primary->kdev->class->pm, sizeof(*new_pm_ops));

        new_pm_ops->restore = new_pm_ops->resume;

        new_pm_ops->thaw = new_pm_ops->resume;

        dev->primary->kdev->class->pm = new_pm_ops;
    }
#endif
    s3g_memset(s3g, 0, sizeof(s3g_card_t));

    dev->dev_private = (void*)s3g;

    pci_set_drvdata(pdev, dev);

    s3g_card_pre_init(s3g, pdev);

    ret = pci_request_regions(pdev, s3g_driver.name);
    if(ret)
    {
        s3g_error("pci_request_regions() failed. ret:%x.\n", ret);
    }

    pci_set_master(pdev);

    /*don't use the vga arbiter*/
#if defined(CONFIG_VGA_ARB)
    vga_set_legacy_decoding(pdev, VGA_RSRC_NONE);
#endif

    ret = s3g_card_init(s3g, pdev);
    if(ret)
    {
        s3g_error("s3g_card_init() failed. ret:0x%x\n", ret);
    }
    
    s3g_info("s3g = %p, s3g->pdev = %p, dev = %p, dev->primary = %p\n", s3g, s3g->pdev,  dev, dev ? dev->primary : NULL);

    return ret;
}

#if 0
static void s3g_drm_unload(struct drm_device *dev)
{
    s3g_card_t *s3g = dev->dev_private;

    if(s3g->s3gfb_enable)
    {
#ifdef CONFIG_FB
        s3g_fb_shutdown(s3g);
#endif
    }
}
#endif 

static int s3g_drm_open(struct drm_device *dev, struct drm_file *file)
{
    s3g_card_t *s3g = NULL;
    s3g_file_t *priv;
    int minor;
    int err = -ENODEV;

    minor = file->minor->index;

    if(!((minor >= 0) && (minor < S3G_MAX_CARDS)))
    {
        return err;
    }

#ifdef S3G_PCIE_BUS
    s3g = &s3g_cards[minor];
#else
    s3g = &s3g_cards[0];
#endif

    if (!s3g->adapter)
    {
        return err;
    }

    priv = s3g_calloc(sizeof(s3g_file_t));
    if (!priv)
        return -ENOMEM;

    file->driver_priv = priv;
    priv->filp  = file->filp;
    priv->card  = s3g;

    priv->lock  = s3g_create_mutex();

    init_waitqueue_head(&priv->wait_queue);
    return 0;
}

static void s3g_drm_preclose(struct drm_device *dev, struct drm_file *file)
{
    s3g_file_t *priv = file->driver_priv;
    s3g_card_t *s3g  = priv->card;

    if (priv->wait_events)
    {
        s3g_remove_wait(s3g, priv);
    }

    if(priv->hold_lock)
    {
        s3g_mutex_unlock(s3g->lock);
    }

    if(priv->gpu_device)
    {
        s3g_cleanup_2d_screen(s3g, priv);

        if(s3g->debugfs_dev)
        {
            s3g_debugfs_remove_node(s3g->debugfs_dev, priv->gpu_device);
        }

        krnl_final_cleanup(s3g->adapter, priv->gpu_device);
        priv->gpu_device = 0;
    }

    if (priv->need_cleanup_drawable)
    {
        s3g_cleanup_drawable(priv);
    }

    s3g_destroy_mutex(priv->lock);
    s3g_free(priv);

    file->driver_priv = NULL;
}

static void s3g_drm_postclose(struct drm_device *dev, struct drm_file *file)
{
}

static int s3g_drm_device_is_agp(struct drm_device * dev)
{
    return 0;
}

static int s3g_mmap(struct file *filp, struct vm_area_struct *vma)
{
    int ret = 0;
#ifndef __frv__
    struct drm_file *file = filp->private_data;
    s3g_file_t     *priv = file->driver_priv;
    s3g_card_t     *card = priv->card;
    s3g_map_argu_t *map  = priv->map;
    s3g_map_argu_t map_argu;

    if(map == NULL)
    {
        bus_config_t bus_config = {0};

        unsigned long offset = vma->vm_pgoff << PAGE_SHIFT;

        s3g_get_bus_config(card->pdev, &bus_config);

        map_argu.flags.mem_type   = S3G_SYSTEM_IO;

        if((offset >= bus_config.mem_start_addr[0]) &&
           (offset <  bus_config.mem_end_addr[0]))
        {
            /* map fb as wc */
            map_argu.flags.cache_type = S3G_MEM_WRITE_COMBINED;
        }
        else
        {
            /* map mmio reg as uc*/
            map_argu.flags.cache_type = S3G_MEM_UNCACHED;
        }

        map = &map_argu;
    }

    switch(map->flags.mem_type)
    {
        case S3G_SYSTEM_IO:
            ret = s3g_map_system_io(vma, map);
            break;

        case S3G_SYSTEM_RAM:
        //case S3G_SYSTEM_RAM_DYNAMIC:
            ret = s3g_map_system_ram(vma, map);
            break;

        default:
            s3g_error("%s, unknown memory map type", __func__);
            ret = -1;
            break;
    }
#endif

    return ret;
}

static long s3g_unlocked_ioctl(struct file *filp, unsigned int cmd, unsigned long arg)
{
    long ret;
    unsigned int nr = S3G_IOCTL_NR(cmd);
    unsigned int type = S3G_IOCTL_TYPE(cmd);
    struct drm_file *file_priv = filp->private_data;
    struct drm_device *dev = file_priv->minor->dev;

    if (type == S3G_IOCTL_BASE)
    {
        if (drm_device_is_unplugged(dev))
            return -ENODEV;

        ret = (long)(s3g_ioctls[nr](file_priv->driver_priv, cmd, arg));

        return ret;
    }
    else
    {
        return drm_ioctl(filp, cmd, arg);
    }
}

#if defined(__x86_64__) && defined(HAVE_COMPAT_IOCTL)
static long s3g_compat_ioctl(struct file *filp, unsigned int cmd, unsigned long arg)
{
    long ret;
    unsigned int nr = S3G_IOCTL_NR(cmd);
    unsigned int type = S3G_IOCTL_TYPE(cmd);
    struct drm_file *file_priv = filp->private_data;
    struct drm_device *dev = file_priv->minor->dev;

    if (type == S3G_IOCTL_BASE)
    {
        if (drm_device_is_unplugged(dev))
        {
            ret = -ENODEV;
        }
        else
        {
            ret = (long)(s3g_ioctls_compat[nr](file_priv->driver_priv, cmd, arg));
        }
    }
    else
    {
        if (nr < DRM_COMMAND_BASE) 
        {
            ret = drm_compat_ioctl(filp, cmd, arg);
        } 
        else 
        {
            ret = drm_ioctl(filp, cmd, arg);
        }
    }

    return ret;
}
#endif

static unsigned int s3g_poll(struct file *filp, struct poll_table_struct *wait)
{
    struct drm_file *file_priv = filp->private_data;
    s3g_file_t *priv = file_priv->driver_priv;

    unsigned int mask = 0;

    poll_wait(filp, &priv->wait_queue, wait);

    if(priv->wait_events & priv->curr_events)
    {
        mask = POLLIN;
    }

    return mask;
} 

static struct file_operations s3g_drm_fops = {
    .owner      = THIS_MODULE,
    .open       = drm_open,
    .release    = drm_release,
    .unlocked_ioctl = s3g_unlocked_ioctl,
#if defined(__x86_64__) && defined(HAVE_COMPAT_IOCTL)
    .compat_ioctl = s3g_compat_ioctl,
#endif
    .mmap       = s3g_mmap,
    .read       = drm_read,
    .poll       = s3g_poll,
    .llseek     = noop_llseek,
};

static struct drm_driver s3g_drm_driver = {
    .driver_features    = 0,
    .load               = s3g_drm_load,
    .open               = s3g_drm_open,
//    .lastclose          = s3g_drm_lastclose,
    .preclose           = s3g_drm_preclose,
    .postclose          = s3g_drm_postclose,

#if LINUX_VERSION_CODE < KERNEL_VERSION(4,9,0)
    .suspend            = s3g_drm_suspend,
    .resume             = s3g_drm_resume,
#endif
    .device_is_agp      = s3g_drm_device_is_agp,
//    .master_create      = s3g_drm_master_create,
//    .master_destroy     = s3g_drm_master_destroy,
    .fops               = &s3g_drm_fops,

    .name               = DRIVER_NAME,
    .desc               = DRIVER_DESC,
    .date               = DRIVER_DATE,
    .major              = DRIVER_MAJOR,
    .minor              = DRIVER_MINOR,
    .patchlevel         = DRIVER_PATCHLEVEL,
};

static int s3g_pcie_probe(struct pci_dev *pdev, const struct pci_device_id *ent)
{
#if LINUX_VERSION_CODE > KERNEL_VERSION(3,10,52)

    INIT_LIST_HEAD(&s3g_drm_driver.legacy_dev_list);//init list manually, drm_get_pci_dev will use this list
    return drm_get_pci_dev(pdev, ent, &s3g_drm_driver);
#else
    return 0;
#endif
}

#if LINUX_VERSION_CODE >= KERNEL_VERSION(3,10,52)
static void __exit s3g_pcie_cleanup(struct pci_dev *pdev)
{
    struct drm_device *dev = pci_get_drvdata(pdev);
    s3g_card_t *s3g = dev->dev_private;
    
    drm_put_dev(dev);

    if(s3g == NULL) return;

    s3g_card_deinit(s3g);

    pci_set_drvdata(pdev, NULL);

    pci_release_regions(pdev);
}
#endif

static int s3g_pmops_suspend(struct device *dev)
{
    struct pci_dev *pdev = to_pci_dev(dev);

    s3g_info("pci device(vendor:0x%X, device:0x%X) pm suspend\n", pdev->vendor, pdev->device);

#if LINUX_VERSION_CODE >= KERNEL_VERSION(4,9,0)
    s3g_drm_suspend(pci_get_drvdata(pdev), PMSG_SUSPEND);
#endif

    
    return 0;
}
static int s3g_pmops_resume(struct device *dev)
{
    struct pci_dev *pdev = to_pci_dev(dev);

    s3g_info("pci device(vendor:0x%X, device:0x%X) pm resume\n", pdev->vendor, pdev->device);

#if LINUX_VERSION_CODE >= KERNEL_VERSION(4,9,0)
    s3g_drm_resume(pci_get_drvdata(pdev));
#endif

    
    return 0;
}

static int s3g_pmops_freeze(struct device *dev)
{
    struct pci_dev *pdev = to_pci_dev(dev);

    s3g_info("pci device(vendor:0x%X, device:0x%X) pm freeze\n", pdev->vendor, pdev->device);

#if LINUX_VERSION_CODE >= KERNEL_VERSION(4,9,0)
    s3g_drm_suspend(pci_get_drvdata(pdev), PMSG_FREEZE);
#endif
    
    return 0;
}

static int s3g_pmops_thaw(struct device *dev)
{
    struct pci_dev *pdev = to_pci_dev(dev);

    s3g_info("pci device(vendor:0x%X, device:0x%X) pm thaw\n", pdev->vendor, pdev->device);

#if LINUX_VERSION_CODE >= KERNEL_VERSION(4,9,0)
    s3g_drm_resume(pci_get_drvdata(pdev));
#endif

    
    return 0;
}

static int s3g_pmops_poweroff(struct device *dev)
{
    struct pci_dev *pdev = to_pci_dev(dev);

    s3g_info("pci device(vendor:0x%X, device:0x%X) pm poweroff\n", pdev->vendor, pdev->device);
    
    return 0;
}

static int s3g_pmops_restore(struct device *dev)
{
    struct pci_dev *pdev = to_pci_dev(dev);

    s3g_info("pci device(vendor:0x%X, device:0x%X) pm restore\n", pdev->vendor, pdev->device);

#if LINUX_VERSION_CODE >= KERNEL_VERSION(4,9,0)
    s3g_drm_resume(pci_get_drvdata(pdev));
#endif

    
    return 0;
}

static int s3g_pmops_runtime_suspend(struct device *dev)
{
    struct pci_dev *pdev = to_pci_dev(dev);

    s3g_info("pci device(vendor:0x%X, device:0x%X) pm runtime_suspend\n", pdev->vendor, pdev->device);
    
    return 0;
}

static int s3g_pmops_runtime_resume(struct device *dev)
{
    struct pci_dev *pdev = to_pci_dev(dev);

    s3g_info("pci device(vendor:0x%X, device:0x%X) pm runtime_resume\n", pdev->vendor, pdev->device);
    
    return 0;
}
static int s3g_pmops_runtime_idle(struct device *dev)
{
    struct pci_dev *pdev = to_pci_dev(dev);

    s3g_info("pci device(vendor:0x%X, device:0x%X) pm runtime_idle\n", pdev->vendor, pdev->device);
    
    return 0;
}

static const struct dev_pm_ops s3g_pm_ops = {
    .suspend = s3g_pmops_suspend,
    .resume = s3g_pmops_resume,
    .freeze = s3g_pmops_freeze,
    .thaw = s3g_pmops_thaw,
    .poweroff = s3g_pmops_poweroff,
    .restore = s3g_pmops_restore,
    .runtime_suspend = s3g_pmops_runtime_suspend,
    .runtime_resume = s3g_pmops_runtime_resume,
    .runtime_idle = s3g_pmops_runtime_idle,
};

static struct pci_driver s3g_driver =
{
    .name = "zx",
    .id_table = pciidlist,
    .probe = s3g_pcie_probe,
#if LINUX_VERSION_CODE < KERNEL_VERSION(3,10,52)
   //.remove   = __devexit_p(s3g_pcie_cleanup),
   .remove   = NULL,
#else
    .remove   = __exit_p(s3g_pcie_cleanup),
#endif
    .driver.pm = &s3g_pm_ops,
};


/*************************************** PCI functions ********************************************/

int s3g_get_command_status16(void *dev, unsigned short *command)
{
    return pci_read_config_word((struct pci_dev*)dev, 0x4, command);
}

int s3g_get_command_status32(void *dev, unsigned int *command)
{
    return pci_read_config_dword((struct pci_dev*)dev, 0x4, command);
}

int s3g_write_command_status16(void *dev, unsigned short command)
{
    return pci_write_config_word((struct pci_dev*)dev, 0x4, command);
}

int s3g_write_command_status32(void *dev, unsigned int command)
{
    return pci_write_config_dword((struct pci_dev*)dev, 0x4, command);
}


int s3g_get_bar1(void *dev, unsigned int *bar1)
{
    return pci_read_config_dword((struct pci_dev*)dev, 0x14, bar1);
}

int s3g_get_rom_save_addr(void *dev, unsigned int *romsave)
{
    return pci_read_config_dword((struct pci_dev*)dev, 0x30, romsave);
}

int s3g_write_rom_save_addr(void *dev, unsigned int romsave)
{
    return pci_write_config_dword((struct pci_dev*)dev, 0x30, romsave);
}

unsigned long s3g_get_rom_start_addr(void *dev)
{
    return pci_resource_start((struct pci_dev*)dev, 6);
}

int s3g_get_platform_config(void *dev, const char* config_name, int *buffer, int length)
{
    return 0;
}

#define DEVICE_MASK       0xFFFF
#define DEVICE_CHX001     0x3A03

void s3g_get_bus_config(void *dev, bus_config_t *bus)
{
    struct pci_dev *pdev = dev;

    pci_read_config_word(pdev, 0x2,  &bus->device_id);
    pci_read_config_word(pdev, 0x0,  &bus->vendor_id);
    pci_read_config_word(pdev, 0x4,  &bus->command);
    pci_read_config_word(pdev, 0x6,  &bus->status);
    pci_read_config_byte(pdev, 0x8,  &bus->revision_id);
    pci_read_config_byte(pdev, 0x9,  &bus->prog_if);
    
    pci_read_config_byte(pdev, 0xa,  &bus->sub_class);
    pci_read_config_byte(pdev, 0xb,  &bus->base_class);
    pci_read_config_byte(pdev, 0xc,  &bus->cache_line_size);
    pci_read_config_byte(pdev, 0xd,  &bus->latency_timer);
    pci_read_config_byte(pdev, 0xe,  &bus->header_type);
    pci_read_config_byte(pdev, 0xf,  &bus->bist);
    pci_read_config_word(pdev, 0x2c, &bus->sub_sys_vendor_id);    
    pci_read_config_word(pdev, 0x2e, &bus->sub_sys_id);
    pci_read_config_word(pdev, 0x52, &bus->link_status);

    s3g_info("bus_command: 0x%x.\n", bus->command);

    //pci_write_config_word(pdev, 0x4,  7);

    bus->reg_start_addr[0] = pci_resource_start(pdev, 0);

#ifdef VIDEO_ONLY_FPGA
    if(bus->device_id == 0x7011)
    {
        s3g_info("device 0x7011 hacked as elite2000.\n");
        bus->device_id = 0x330F;
    }
#endif

#ifdef VIDEO_PCIE_IO_REMAP
    bus->mem_start_addr[0] = pci_resource_start(pdev, 0) + 0x30000; 
#else
    bus->mem_start_addr[0] = pci_resource_start(pdev, 1);
#endif

    bus->reg_start_addr[2] = 0;
    bus->reg_start_addr[3] = 0;
    bus->reg_start_addr[4] = 0;


    bus->reg_end_addr[0]   = pci_resource_end(pdev, 0);

#ifdef VIDEO_PCIE_IO_REMAP
    bus->mem_end_addr[0]   = pci_resource_end(pdev, 0);
#else
    bus->mem_end_addr[0]   = pci_resource_end(pdev, 1);
#endif

    bus->reg_end_addr[2]   = 0;
    bus->reg_end_addr[3]   = 0;
    bus->reg_end_addr[4]   = 0;

#ifdef VIDEO_PCIE_IO_REMAP
    bus->reg_start_addr[0] = 0xc0000000;
    bus->reg_end_addr[0]   = 0xc0000000 + 0x10000;

    bus->reg_start_addr[1] = 0xc0000000 + 0x00010000;
    bus->reg_end_addr[1]   = 0xc0000000 + 0x00010000 + 0x10000;

    bus->reg_start_addr[3] = 0xc0000000 + 0x00020000;
    bus->reg_end_addr[3]   = 0xc0000000 + 0x00020000 + 0x10000 ;
#endif

   if((bus->device_id  & DEVICE_MASK) == DEVICE_CHX001 )
   {
	    void *mmio = s3g_ioremap(bus->reg_start_addr[0], bus->reg_end_addr[0] - bus->reg_start_addr[0]);

	    if(mmio != NULL)
	    {
	        bus->revision_id = s3g_read32(mmio + 0x8A82)&0xFF;
	        s3g_info("*******revision id:%x\n",bus->revision_id);
	        s3g_iounmap(mmio);
	    }
	    else
	    {
	        s3g_error("map io address failed: phys start: %x, phys end: %x.\n", bus->reg_start_addr[0], bus->reg_end_addr[0]);
	    }
   }


}

void s3g_init_bus_id(s3g_card_t *s3g)
{
    struct pci_dev *pdev = s3g->pdev;

    int pci_domain = 0;
    int pci_bus    = pdev->bus->number;
    int pci_slot   = PCI_SLOT(pdev->devfn);
    int pci_func   = PCI_FUNC(pdev->devfn);

    s3g->len = snprintf(s3g->busId, 40, "pci:%04x:%02x:%02x.%d", pci_domain, pci_bus, pci_slot, pci_func);
}

int s3g_register_driver(void)
{
    int ret = 0;
    
    ret = pci_register_driver(&s3g_driver);//register driver first,  register drm device during pci probe
#if LINUX_VERSION_CODE <= KERNEL_VERSION(3,10,52)
    ret = drm_pci_init(&s3g_drm_driver, &s3g_driver);
#endif

    return ret;
}

void s3g_unregister_driver(void)
{

#if LINUX_VERSION_CODE <= KERNEL_VERSION(3,10,52)
    drm_pci_exit(&s3g_drm_driver, &s3g_driver);
#endif

    pci_unregister_driver(&s3g_driver);
}

int s3g_register_interrupt(s3g_card_t *s3g, void *isr)
{
#ifdef S3G_GPIO_IRQ
     int ret;

     ret = gpio_request(gpio_for_pcie, "gpio_pcie");
     if (ret < 0)
     {
        s3g_error("gpio request failed\n");
        return ret;
     }

     ret = gpio_direction_input(gpio_for_pcie);
     if (ret < 0)
     {
        s3g_error("gpio setup input direction failed\n");
        return ret;
     }

     return request_irq(gpio_to_irq(gpio_for_pcie),
           (irqreturn_t (*)(int, void*))isr, IRQF_DISABLED | IRQF_TRIGGER_RISING, "s3g", s3g);
#else
    if(s3g->msi_enable)
    {
        s3g->msi_enable = pci_enable_msi(s3g->pdev) ? FALSE : TRUE;
        if(s3g->msi_enable)
        {
            krnl_enable_msi(s3g->adapter);
        }
    }

    return request_irq(((struct pci_dev*)s3g->pdev)->irq,
        (irqreturn_t (*)(int, void*))isr, IRQF_SHARED, "zx", s3g);
#endif
}

void s3g_unregister_interrupt(s3g_card_t *s3g)
{
#ifdef S3G_GPIO_IRQ
    free_irq(gpio_to_irq(gpio_for_pcie), s3g);
    gpio_free(gpio_for_pcie);
#else
    free_irq(((struct pci_dev*)s3g->pdev)->irq, s3g);

    if(s3g->msi_enable)
    {
        pci_disable_msi((struct pci_dev*)s3g->pdev);
    } 
#endif
}
