/* 
* Copyright 1998-2014 VIA Technologies, Inc. All Rights Reserved.
* Copyright 2001-2014 S3 Graphics, Inc. All Rights Reserved.
* Copyright 2013-2014 Shanghai Zhaoxin Semiconductor Co., Ltd. All Rights Reserved.
*
* This file is part of zx.ko
* 
* zx.ko is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* zx.ko is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "s3g_screen.h"
#include "os_interface.h"
#include "kernel_interface.h"

#define MAX_SERVERS   32
#define MAX_DRAWABLES 1024

#define WIDTH_4K 3840

static server_info_t  s3g_servers[MAX_SERVERS]     = {{0},};
static s3g_drawable_t s3g_drawables[MAX_DRAWABLES] = {{0},};

framebuffer_t* s3g_get_frame_buffer(s3g_card_t *card, int server_index, int screen_index)
{
    return s3g_servers[server_index].frame_buffers[card->index][screen_index];
}

void s3g_set_current_fb(s3g_card_t *card, int server_index, int screen_index, framebuffer_t *fb)
{
    s3g_servers[server_index].frame_buffers[card->index][screen_index] = fb;
}

static int s3g_drawable_fullscreen(s3g_card_t *card, s3g_get_drawsize_t *drawRec)
{
    framebuffer_t *fb;

    fb = s3g_get_frame_buffer(card, drawRec->server_index, drawRec->scrnIndex);

    if(fb)
    {
        if(fb->width        == (drawRec->rect.x2 - drawRec->rect.x1) &&
           fb->height       == (drawRec->rect.y2 - drawRec->rect.y1) &&
           drawRec->rect.x1 == 0 && drawRec->rect.y1 == 0 )
        {
            return 1;
        }
    }

    return 0;
}

static framebuffer_t* s3g_create_framebuffer(s3g_card_t *card, s3g_2d_screen_t *screen)
{
    framebuffer_t *fb = NULL;
    framebuffer_t *fb_shadow = NULL;
    s3g_create_allocation_t    create = {0};
    s3g_query_info_t info = {0};

    fb               = s3g_calloc(sizeof(framebuffer_t));
    fb->width        = screen->width;
    fb->height       = screen->height;
    fb->screen_index = screen->scrnIndex;
    fb->server_index = screen->server_index;
    fb->depth        = screen->bpp;
    fb->use_rxa      = screen->use_rxa;
    fb->duoview      = screen->bDuoView;
    fb->adapter      = card->adapter;
    fb->hdevice      = screen->device;
    fb->tiled        = TRUE;

    info.type = S3G_QUERY_LOCAL_ALLOCATION_MAX_SIZE;
    krnl_query_info(card->adapter, &info);
    
    if (screen->width * screen->height > info.value)
    {
        fb->tiled = TRUE;
    }

    create.device      = screen->device;
    create.width       = screen->width;
    create.height      = screen->height;
    create.format      = (screen->bpp == 32) ? S3G_FORMAT_B8G8R8A8_UNORM : S3G_FORMAT_B5G6R5_UNORM;
    create.tiled       = fb->tiled;
    if(/*screen->width >= WIDTH_4K || */screen->bpp == 16)
    {
        create.unpagable   = FALSE;
        create.usage_mask  = S3G_USAGE_DATA_BUFFER;
        create.access_hint = S3G_ACCESS_CPU_ALMOST;
    }
    else
    {
        create.unpagable   = TRUE;
        create.usage_mask  = S3G_USAGE_DISPLAY_SURFACE | S3G_USAGE_RENDER_TARGET | S3G_USAGE_TEXTURE_BUFFER;
        create.access_hint = S3G_ACCESS_GPU_ALMOST;
        create.primary     = 1;
    }

    krnl_create_allocation(card->adapter, &create);

    fb->hAllocation = create.allocation;
    fb->fbOffset    = create.gpu_virt_addr;
    fb->pitch       = create.pitch;
    fb->gbd         = create.gbd;
    fb->tiled       = create.tiled;

    if(/*screen->width >= WIDTH_4K || */screen->bpp == 16)
    {
        fb_shadow               = s3g_calloc(sizeof(framebuffer_t));
        fb_shadow->width        = screen->width;
        fb_shadow->height       = screen->height;
        fb_shadow->screen_index = screen->scrnIndex;
        fb_shadow->server_index = screen->server_index;
        fb_shadow->depth        = screen->bpp;
        fb_shadow->use_rxa      = screen->use_rxa;
        fb_shadow->duoview      = screen->bDuoView;
        fb_shadow->adapter      = card->adapter;
        fb_shadow->hdevice      = screen->device;
        fb_shadow->tiled        = FALSE;

        create.device      = screen->device;
        create.width       = screen->width;
        create.height      = screen->height;
        create.format      = (screen->bpp == 32) ? S3G_FORMAT_B8G8R8A8_UNORM : S3G_FORMAT_B5G6R5_UNORM;
        create.tiled       = fb_shadow->tiled;
        create.unpagable   = TRUE;
        create.primary     = 1;
        create.usage_mask  = S3G_USAGE_DISPLAY_SURFACE | S3G_USAGE_RENDER_TARGET | S3G_USAGE_FRAMEBUFFER | S3G_USAGE_TEXTURE_BUFFER;
        create.access_hint = S3G_ACCESS_GPU_ALMOST;

        krnl_create_allocation(card->adapter, &create);

        fb_shadow->hAllocation = create.allocation;
        fb_shadow->fbOffset    = create.gpu_virt_addr;
        fb_shadow->pitch       = create.pitch;
        fb_shadow->gbd         = create.gbd;
        fb_shadow->tiled       = create.tiled;
        
        krnl_set_fb_shadow_for_adapter(card->adapter, fb->hAllocation);
    }

    screen->hAllocation = create.allocation;
    screen->fbOffset    = create.gpu_virt_addr;
    screen->pitch       = create.pitch;
    screen->gbd         = create.gbd;
    screen->tiled       = create.tiled;

    s3g_info("create_framebuffer(alloc:%x,width:%d,height:%d,pitch:%d,bpp:%d,tiled:%d,segment_id:%d)\n",
    create.allocation, create.width, create.height, create.pitch, create.bit_cnt, create.tiled, create.segment_id);

    if(/*screen->width >= WIDTH_4K || */screen->bpp == 16)
    {
        s3g_set_current_fb(card, fb_shadow->server_index, fb_shadow->screen_index, fb_shadow);
        s3g_servers[fb_shadow->server_index].frame_shadow_buffers[card->index][fb_shadow->screen_index] = fb;
        return fb_shadow;
    }
    else
    {
        s3g_set_current_fb(card, fb->server_index, fb->screen_index, fb);
        return fb;
    }
}

static int s3g_destroy_framebuffer(s3g_card_t *card, framebuffer_t *fb)
{
    framebuffer_t           *current_fb = NULL;
    framebuffer_t           *current_fb_shadow = NULL;
    s3g_destroy_allocation_t destroy = {0};

    current_fb = s3g_get_frame_buffer(card, fb->server_index, fb->screen_index);

    if(current_fb == fb)
    {
        s3g_set_current_fb(card, fb->server_index, fb->screen_index, NULL);
    }

    destroy.device     = fb->hdevice;
    destroy.allocation = fb->hAllocation;

    krnl_destroy_allocation(card->adapter, &destroy);

    fb->priv = NULL;
    
    if(/*fb->width >= WIDTH_4K || */fb->depth == 16)
    {
        current_fb_shadow = s3g_servers[fb->server_index].frame_shadow_buffers[card->index][fb->screen_index];
        
        if(current_fb_shadow)
        {
            destroy.device     = current_fb_shadow->hdevice;
            destroy.allocation = current_fb_shadow->hAllocation;
            krnl_destroy_allocation(card->adapter, &destroy);

            s3g_servers[fb->server_index].frame_shadow_buffers[card->index][fb->screen_index] = NULL;

            current_fb_shadow->priv = NULL;
            s3g_free(current_fb_shadow);

            krnl_set_fb_shadow_for_adapter(card->adapter, 0);
        }
    }

    s3g_free(fb);

    return 0;
}

void s3g_init_2d_screen(s3g_card_t *card, s3g_file_t *priv, s3g_2d_screen_t *screen)
{
    framebuffer_t *fb;
    framebuffer_t *old_fb;

    old_fb = s3g_get_frame_buffer(card, screen->server_index, screen->scrnIndex);

    if (old_fb &&
        old_fb->width  == screen->width && 
        old_fb->height == screen->height &&
        old_fb->depth  == screen->bpp)
    {        
        screen->hAllocation = old_fb->hAllocation;
        screen->fbOffset    = old_fb->fbOffset;
        screen->pitch       = old_fb->pitch;
        screen->gbd         = old_fb->gbd;
        screen->tiled       = old_fb->tiled;
        return; 
    }

    fb = s3g_create_framebuffer(card, screen);

    fb->priv = priv;
    
    if(old_fb)
    {
        /*keep the rot_allocation till the user layer set or clear it*/
        /* need check one issue: consider connected two monitor, all rotate, 
         * OK, unplug one monitor, and set the left normal, do this condition 
         * still can get right info, if still kepp old value. 
         */

        fb->rot_allocation[0] = old_fb->rot_allocation[0];
        fb->rot_allocation[1] = old_fb->rot_allocation[1];       
        
        //krnl_notify_resize_fb(card->adapter, old_fb->hAllocation, 0);
        /* Here the fb_allocation should be updated to new allocation, 
        * or when running "xrandr --output CRT1 --pos 0x64" with clone
        * mode, the client will only requst to set IGA0 start address, 
        * and the IGA1 will not be updated.
        */
        krnl_notify_resize_fb(card->adapter, old_fb->hAllocation, fb->hAllocation);

        s3g_destroy_framebuffer(card, old_fb);
    }
 
}

void s3g_deinit_2d_screen(s3g_card_t *card, s3g_2d_screen_t *screen)
{
    framebuffer_t *fb;

    fb = s3g_get_frame_buffer(card, screen->server_index, screen->scrnIndex);

    if(fb)
    {
        krnl_notify_resize_fb(card->adapter, fb->hAllocation, 0);

        s3g_destroy_framebuffer(card, fb);
    }
}

void s3g_cleanup_drawable(s3g_file_t *priv)
{
    int drawId, i;

    for(drawId = 0; drawId < MAX_DRAWABLES; drawId++)
    {
        if (s3g_drawables[drawId].refCount > 0)
        {
            for(i = 0; i < MAX_SCREENS; i++)
            {
                if(s3g_drawables[drawId].server_index == priv->server_index)
                {
                    if (s3g_drawables[drawId].clipNotify[i].clipRects)
                    {
                        s3g_free(s3g_drawables[drawId].clipNotify[i].clipRects);
                    }
                    s3g_memset(&s3g_drawables[drawId], 0, sizeof(s3g_drawable_t));
                }
            }
        }
    }
}

void s3g_cleanup_2d_screen(s3g_card_t *card, s3g_file_t *priv)
{
    framebuffer_t *fb;
    int i, j;

    for(i = 0; i < MAX_SERVERS; i++)
    {
       for(j = 0; j < MAX_SCREENS; j++)
       {
           fb = s3g_get_frame_buffer(card, i, j);
           if((fb != NULL) && (fb->priv == priv))
           {
               krnl_notify_resize_fb(card->adapter, fb->hAllocation, 0);
               s3g_destroy_framebuffer(card, fb);
           }
       }
    }
}

void s3g_set_server_info(s3g_card_t *card, int server_index, int xinerama)
{
    server_info_t                 *server_info;

    server_info = &s3g_servers[server_index];

    server_info->xinerama = xinerama;   
    
}

int s3g_is_xinerama(s3g_card_t *card, int server_index)
{
    return s3g_servers[server_index].xinerama;
}

void s3g_map_iga_to_screen(s3g_card_t *card, int *screens, int server_index)
{
    framebuffer_t               *fb;
    int                         i;
    int                         j;
    unsigned int                handle;

    for(i = 0; i < MAX_CORE_CRTCS; i++)
    {
        handle = krnl_get_iga_allocation(card->adapter, i);

        for(j = 0; j < MAX_SCREENS; j++)
        {
            fb = s3g_get_frame_buffer(card, server_index, j);
            
            if(fb && fb->hAllocation == handle)
            {
                break;
            }
        }

        if(j < MAX_SCREENS)
        {
            screens[i] = j;
        }
        else
        {
            screens[i] = -1;
        }
    }
}

void s3g_map_screen_to_igas(s3g_card_t *card, int *igas, int server_index, int screen_index)
{
    server_info_t               *server_info;
    framebuffer_t               *fb;
    int                         i;
    int                         j;
    unsigned int                handle;

    server_info = &s3g_servers[server_index];

    for(i = 0; i < MAX_CORE_CRTCS; i++)
    {
        handle = krnl_get_iga_allocation(card->adapter, i);

        if(server_info->xinerama)
        {
            for(j = 0; j < MAX_SCREENS; j++)
            {
                fb = s3g_get_frame_buffer(card, server_index, j);
            
                if(fb && fb->hAllocation == handle)
                {
                    break;
                }
            }

            if(j < MAX_SCREENS)
            {
                igas[i] = i;
            }
            else
            {
                igas[i] = -1;
            }
        }
        else
        {
            fb = s3g_get_frame_buffer(card, server_index, screen_index);

            if(fb->hAllocation == handle)
            {
                igas[i] = i;
            }
            else
            {
                igas[i] = -1;
            }
        }
    }
}

int s3g_get_display_info(s3g_card_t *card, s3g_display_info_t *display_info)
{
    framebuffer_t               *fb;
    server_info_t               *server_info;

    fb = s3g_get_frame_buffer(card, display_info->server_index, display_info->screen_index);

    if(!fb)
    {
        return -1;
    }

    server_info = &s3g_servers[display_info->server_index];

    display_info->bDuoView   = fb->duoview;
    display_info->bXinerama  = server_info->xinerama;
    display_info->bufBpp     = fb->depth/8;
    display_info->bUseRxa    = fb->use_rxa;

#ifndef S3G_HW_NULL
    s3g_map_iga_to_screen(card, display_info->iga2scrnIndex, display_info->server_index);
#endif

    return 0;
    
}

void s3g_clip_notify(s3g_card_t *card, s3g_clipnotify_t *clip)
{   
    if(clip->scrnIndex > MAX_SCREENS)
    {
        s3g_error("s3g_clip_notify, scrnIndex > MAX_SCFREENS!\n");
        return;
    } 
 
    if(clip->kdrawId && s3g_drawables[clip->kdrawId].refCount)
    {
        int            i  = clip->kdrawId;
        framebuffer_t *fb = s3g_get_frame_buffer(card, s3g_drawables[i].server_index, clip->scrnIndex);

        if(!clip->onlyChangeKernelBufferHandle)
        {
            s3g_drawables[i].clipNotify[clip->scrnIndex].x = clip->x;
            s3g_drawables[i].clipNotify[clip->scrnIndex].y = clip->y;
            s3g_drawables[i].clipNotify[clip->scrnIndex].screen_x = clip->screen_x;
            s3g_drawables[i].clipNotify[clip->scrnIndex].screen_y = clip->screen_y;
            s3g_drawables[i].clipNotify[clip->scrnIndex].width  = clip->width;
            s3g_drawables[i].clipNotify[clip->scrnIndex].height = clip->height;
            s3g_drawables[i].clipNotify[clip->scrnIndex].numClipRects = clip->numClipRects; 
            s3g_drawables[i].clipNotify[clip->scrnIndex].hAllocation  = clip->hAllocation ? 
                                                                        clip->hAllocation : fb->hAllocation; 
            s3g_drawables[i].clipNotify[clip->scrnIndex].iga_mask     = 
                krnl_get_iga_mask_per_allocation(card->adapter, fb->hAllocation);

            /* Take care about when the draw clip rectangle increase */
            if ((s3g_drawables[i].clipNotify[clip->scrnIndex].clipRectsVol < clip->numClipRects) &&
                s3g_drawables[i].clipNotify[clip->scrnIndex].clipRects) {
                s3g_free(s3g_drawables[i].clipNotify[clip->scrnIndex].clipRects);
                s3g_drawables[i].clipNotify[clip->scrnIndex].clipRects = 0;
                s3g_drawables[i].clipNotify[clip->scrnIndex].clipRectsVol = 0;
            }

            if(clip->numClipRects)
            {
                if(s3g_drawables[i].clipNotify[clip->scrnIndex].clipRects == 0)
                {
                    s3g_drawables[i].clipNotify[clip->scrnIndex].clipRects = s3g_malloc(sizeof(s3g_clip_rect_t) * clip->numClipRects);
                    s3g_drawables[i].clipNotify[clip->scrnIndex].clipRectsVol = clip->numClipRects; 
                }
                s3g_copy_from_user(s3g_drawables[i].clipNotify[clip->scrnIndex].clipRects, clip->clipRects, sizeof(s3g_clip_rect_t)*clip->numClipRects);
            }
        }
        else
        {
            s3g_drawables[i].clipNotify[clip->scrnIndex].hAllocation  = clip->hAllocation ? 
                                                                        clip->hAllocation : fb->hAllocation; 
        }
    }
}


void s3g_video_drawable_damage(s3g_card_t *card, s3g_video_damage_t *damage)
{
    int i, count;
    s3g_video_layer_info_t *pInfo;
    s3g_drawable_t *drawable;

    if (damage->layerCount == 0)
    {
        return;
    }

    pInfo = s3g_malloc(damage->layerCount * sizeof(*pInfo));
    s3g_copy_from_user(pInfo, damage->pLayerInfo, damage->layerCount * sizeof(*pInfo));

    if (damage->opCode == S3G_SET_VIDEO_LAYER_INFO)
    {
        for(i = 0; i < damage->layerCount; i++)
        {
            s3g_assert(pInfo[i].drawId != 0);

            drawable = &(s3g_drawables[pInfo[i].drawId]);
            drawable->is_overlay = pInfo[i].is_overlay;
            drawable->video_layer = pInfo[i].is_video;
            drawable->video_damage = pInfo[i].rect;
            if (pInfo[i].forceblt == 0 || pInfo[i].forceblt == 1)
            {
                drawable->forceblt = pInfo[i].forceblt;
            }
        }
    }
    else if (damage->opCode == S3G_GET_VIDEO_LAYER_INFO)
    {
        count = 0;

        for(i = 0; i < MAX_DRAWABLES && count < damage->layerCount; i++)
        {
            drawable = &(s3g_drawables[i]);
            if (drawable->xId != 0 &&
                drawable->video_layer &&
                drawable->clipNotify[0].hAllocation)
            {
                pInfo[count].drawId = i; 
                pInfo[count].is_overlay = drawable->is_overlay;
                pInfo[count].is_video = 1;
                pInfo[count].forceblt = drawable->forceblt;
                pInfo[count].buffer = drawable->clipNotify[0].hAllocation;
                pInfo[count].rect = drawable->video_damage;
                count++;
            }
        }

        damage->layerCount = count;
        s3g_copy_to_user(damage->pLayerInfo, pInfo, sizeof(*pInfo) * count);
    }
    else
    {
        s3g_assert(0);
    }

    s3g_free(pInfo);
}

void s3g_add_drawable(s3g_card_t *card, s3g_drawable_t *drawable)
{
    unsigned int    xId = drawable->xId;
    int    server_index = drawable->server_index;
    unsigned int drawId = 0;
    int  i = 0,   found = 0;

    for(i = 1; i < MAX_DRAWABLES; i++)
    {
        if((s3g_drawables[i].xId == xId) && (s3g_drawables[i].server_index == server_index))
        {
            s3g_drawables[i].refCount++;
            found = 1;
            drawId = i;
            break;
        }
    }

    if(!found)
    {
        for(i = 1; i < MAX_DRAWABLES; i++)
        {
            if(s3g_drawables[i].xId == 0)
            {
                s3g_drawables[i].server_index = server_index;
                s3g_drawables[i].xId          = xId;
                s3g_drawables[i].refCount++;
                found = 1;
                drawId = i;
                break;
            }
        }
    }

    if(server_index == 0)
    {
        // Nox driver has no clip notify mechanism, so set fb size as drawable size
        framebuffer_t *fb = s3g_get_frame_buffer(card, 0, 0);
        i = drawId;

        s3g_drawables[i].clipNotify[0].x = 0;
        s3g_drawables[i].clipNotify[0].y = 0;
        s3g_drawables[i].clipNotify[0].width  = fb->width;
        s3g_drawables[i].clipNotify[0].height = fb->height;
        s3g_drawables[i].clipNotify[0].numClipRects = 1;

        if(s3g_drawables[i].clipNotify[0].clipRects == 0)
        {
            s3g_drawables[i].clipNotify[0].clipRects = s3g_malloc(sizeof(s3g_clip_rect_t)*1);
        }

        s3g_drawables[i].clipNotify[0].clipRects[0].x1 = 0;
        s3g_drawables[i].clipNotify[0].clipRects[0].y1 = 0;
        s3g_drawables[i].clipNotify[0].clipRects[0].x2 = fb->width;
        s3g_drawables[i].clipNotify[0].clipRects[0].y2 = fb->height;
        s3g_drawables[i].clipNotify[0].iga_mask        = krnl_get_iga_mask_per_allocation(card->adapter, fb->hAllocation);
        s3g_drawables[i].clipNotify[0].hAllocation     = fb->hAllocation;
    }

    drawable->xId = drawId;
}

void s3g_remove_drawable(s3g_card_t *card, int drawId)
{
    int i;

    /* siliently ignore if user mode driver remove more than once */
    if(drawId >= MAX_DRAWABLES || s3g_drawables[drawId].refCount == 0)
    {
        s3g_error("s3g_remove_drawable invalid drawId!\n");
        return;
    }
    
    if((--s3g_drawables[drawId].refCount) == 0)
    {
        for(i = 0; i < MAX_SCREENS; i++)
        {
            if(s3g_drawables[drawId].clipNotify[i].clipRects)
            {
                s3g_free(s3g_drawables[drawId].clipNotify[i].clipRects);
            }
        }
        s3g_memset(&s3g_drawables[drawId], 0, sizeof(s3g_drawable_t));
    }
}

void s3g_get_drawable_info(s3g_card_t *card, s3g_get_drawsize_t *draw_rec)
{
    int   i, server_index;
    unsigned int drawId = draw_rec->drawId;
    int iga_mask        = 0;
    
    framebuffer_t *fb;

    s3g_memset(draw_rec, 0, sizeof(s3g_get_drawsize_t));

    // For samm case
    // Xinerama: drawable size is the same for all screen, so any screen is ok.
    // Individual displays: the clip list and drawable size is just stored acording to the scrnIndex
    // And there is just one entry, so need to search to get it.
    for(i = 0; i < MAX_SCREENS; i++)
    {
        if(s3g_drawables[drawId].clipNotify[i].width != 0)
        {
            draw_rec->drawId       = drawId;
            draw_rec->rect.x1      = s3g_drawables[drawId].clipNotify[i].x;
            draw_rec->rect.y1      = s3g_drawables[drawId].clipNotify[i].y;
            draw_rec->rect.x2      = s3g_drawables[drawId].clipNotify[i].width  + draw_rec->rect.x1;
            draw_rec->rect.y2      = s3g_drawables[drawId].clipNotify[i].height + draw_rec->rect.y1;
            iga_mask               = s3g_drawables[drawId].clipNotify[i].iga_mask;
            break;
        }
    }

    if (i >= MAX_SCREENS)
    {
        return;
    }

    server_index = s3g_drawables[drawId].server_index;
    draw_rec->server_index = server_index;
    draw_rec->scrnIndex = i;
    draw_rec->fullscreen = s3g_drawable_fullscreen(card, draw_rec);
    /* if rotation/xinerama is available, fullscreen mode is not supported */
    fb = s3g_get_frame_buffer(card, server_index, draw_rec->scrnIndex);
    /*Fixme: tmp fix app get drawable info after fb being destroied. e.g. Fangde 4k hdmi&&crt, hot plug hdmi many times will lead this issue*/
    if(!fb)
        return;

    /* if rot_allocation exist, means fb->allocation used as src, can not do flip */
    draw_rec->fullscreen &= (fb->rot_allocation[0] || fb->rot_allocation[1]) ? FALSE : TRUE;

    draw_rec->fullscreen &= !s3g_servers[server_index].xinerama;

    draw_rec->redirect    = (fb->hAllocation != s3g_drawables[drawId].clipNotify[i].hAllocation);

    draw_rec->shadow      = (fb->rot_allocation[0] || fb->rot_allocation[1]) ? TRUE : FALSE;
}

void s3g_dump_clips(s3g_card_t *card)
{
    int i, j, k;
    s3g_iga_virtual_info_t *vinfo = NULL;
    
    s3g_info("Iga VirtualInfo:\n");
    for(i = 0; i < MAX_CORE_CRTCS; i++)
    {
        vinfo = card->iga_virtual_info + i;
        s3g_info("iga%d  active:%d virtualX:%d virtualY:%d Width:%d Height:%d\n",
            vinfo->iga_index, vinfo->active, vinfo->virtualX, vinfo->virtualY, vinfo->width, vinfo->height);
    }

    s3g_info("Hw Drawables:\n");
    for(i = 0; i < MAX_DRAWABLES; i++)
    {
        if (s3g_drawables[i].xId)
        {
            s3g_info("kdrawId:%d xId:0x%x server_index:%d video:%d video_damage:[%d %d %d %d], refCount:%d\n", 
                i, s3g_drawables[i].xId, s3g_drawables[i].server_index, s3g_drawables[i].video_layer, 
                s3g_drawables[i].video_damage.x1, s3g_drawables[i].video_damage.y1, s3g_drawables[i].video_damage.x2, s3g_drawables[i].video_damage.y2,
                s3g_drawables[i].refCount);

            for(j = 0; j < MAX_SCREENS; j++)
            {
                if (s3g_drawables[i].clipNotify[j].hAllocation)
                {
                    s3g_clipnotify_t *clipNotify = &(s3g_drawables[i].clipNotify[j]);

                    s3g_info("  scrn:%d screen_x:%d screen_y:%d x:%d y:%d w:%d h:%d hAllocation:%x\n",
                        j, clipNotify->screen_x, clipNotify->screen_y, clipNotify->x, clipNotify->y, clipNotify->width, clipNotify->height, clipNotify->hAllocation);
                    for(k = 0; k < clipNotify->numClipRects; k++)
                    {
                        s3g_info("  rect:%d [%d %d %d %d]\n", 
                            k,
                            clipNotify->clipRects[k].x1, clipNotify->clipRects[k].y1, 
                            clipNotify->clipRects[k].x2, clipNotify->clipRects[k].y2);
                    }
                }
            }
        }

    }
}

s3g_drawable_t *s3g_get_drawable(unsigned int kdraw)
{
    s3g_drawable_t *drawable = NULL;

    if(kdraw < MAX_DRAWABLES)
    {
        drawable = &s3g_drawables[kdraw];
    }
    return drawable;
}

